#!/usr/bin/perl
#-----------------------------------------------------------------------
#   CMS.cgi - ExSite content-management front-end
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Form;
use ExSite::Page;
use ExSite::Content;
use ExSite::Misc;
use ExSite::Input;
use ExSite::Wizard;
use ExSite::PageBuf;

use vars qw($db %query @pathtable @pathcol $section $page $content $cdata 
	    $nonactions);

@pathtable = ( "section", "page", "content", "content_data" );
@pathcol = ( "title", "title", "name", "content_data_id" );

# actions that are implicitly processed, instead of being passed to DoAction
$nonactions = "map";

&exsite_init;
my $db = new ExSite::Form;

# enable page buffering for better redirect control
$config{page_buffering} = 1;
my $p = new ExSite::PageBuf(title=>"ExSite CMS", headtype=>1);

# resources indicated by a path of section/page/content_object
my @cmspath = split /\//,$ENV{PATH_INFO};
#shift @cmspath;
$section = $cmspath[1];
$page = $cmspath[2];
$content = $cmspath[3];
$cdata = $cmspath[4];

my $CGI = $config{server}->{CGIpath};

my $in = new ExSite::Input;
my %query = %{$in->query("data")};
my %formdata = %{$in->post};
my $select = $query{select};

###### obsolete???
#if ($formdata{reply} && 
#    (($formdata{skip_update} =~ /on/i && 
#      $formdata{submit_action} =~ /next/i) || 
#     $formdata{submit_action} =~ /finish/i)) {
#    $p->location($formdata{reply});
#}

# add CBE menu stylesheet
$p->print("<LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$config{server}{HTMLpath}/_ExSite/js/menu.css\">\n");
$p->print("<div align=center>\n<table width=90%><tr><td>\n");

#if ($db->authorize >= $config{auth}->{root_auth_level} - 1) {
my $access = $db->authorize;
if ($access >= 2) {
    $p->print(&TopMenu);
    if ($query{action} && $query{action} !~ /^$nonactions$/) {
	$p->print(&DoAction($query{action}));
    }
    elsif ($cdata) {
	$p->print(&ManageContentData($cdata));
    }
    elsif ($content) {
	$p->print(&ManageContent($content));
    }
    elsif ($page) {
	$p->print(&ManagePage($page));
    }
    elsif ($ENV{PATH_INFO}) {  # $section might be 0
	$p->print(&ManageSection($section));
    }
    else {
	$p->print(&ManageAllSections);
    }
}
else {
    $p->print("<P CLASS=error>Permission denied.</P>\n");
}

if ($db->{sql_error}) {
    $p->print("<hr>\n",$db->{sql_error});
}

$p->print("</div>\n</td></tr></table>\n");

# dump the page
$p->show;

### end

sub TopMenu {
    my @menupath;
    push @menupath, "<a href=$CGI/CMS.cgi>Top</a>";
    my $ipath = 0;
    my @idpath = @cmspath;
    shift @idpath;
    my $path_info;
    foreach my $id (@idpath) {
	$path_info .= "/$id";
	if ($id) {
	    my $data = $db->fetch($pathtable[$ipath],$id);
	    push @menupath, "<a href=$CGI/CMS.cgi$path_info>$data->{$pathcol[$ipath]}</a>";
	}
	else {
	    push @menupath, "[n/a]";
	}
	$ipath++;
    }
    my $text = join(" > ",@menupath);
    return "<table><tr><td><img src=$config{server}{HTMLpath}/_Modules/CMS/icon.gif height=64 width=64 border=0></td><td>
<table cellspacing=0 cellpadding=0 border=0><tr>
<td><img src=$config{server}{HTMLpath}/_Modules/CMS/textwindow-left.gif height=25 width=12></td>
<td background=$config{server}{HTMLpath}/_Modules/CMS/textwindow-mid.gif>$text</td>
<td><img src=$config{server}{HTMLpath}/_Modules/CMS/textwindow-right.gif height=25 width=12></td>
</tr></table></td></tr></table>\n";
}

sub ManageAllSections {
    # list all sections to be managed
    my $title = "Websites and Website Sections";
    my $uid = $db->my_uid;
    my @sdata = $db->fetch_m2m("section","member",$uid,"member_site_link");
    my $help = "<p>Your websites and site templates are contained in folders at right.  Click on the website you want to open.</p>\n";
    my @tables = $db->{map}->get_mapped_tables;
    my $text;
    if (grep(/^section$/, @tables) == 0) {
	$text = "<h3>Error</h3><p>The Website Editor has not been fully installed on your website.  Please contact your website administrator.</p>\n";
    }
    else {
	$text = &ShowObjects("icon",@sdata);
    }
    return &htmlbox($title,&MenuBar,$help,$text); 
}

sub ManageSection {
    my $sid = shift;
    my ($section,@pdata);
    $section = {};
    if ($sid > 0) {
	$section = $db->fetch("section",$sid);
	@pdata = $db->fetch_child("page","section",$sid);
    }
    my ($text,$help);
    if ($query{action} eq "map") {
	$text = "<H3>Site Map</h3>\n".&MapSection;
	$help .= "<p>This site's pages are shown at left.  Click on the one you want to open.</p>\n";
    }
    else {
	$text = &ShowObjects("icon",@pdata),
	$help .= "<p>This site's pages, templates, and libraries are shown at right.  Click on the one you want to open.</p>\n";
    }
    $help .= "<P>Details of this website:</P>\n".$db->report_row("section",$section)."\n";
    return &htmlbox("Website: $section->{title}",
		    &MenuBar(\@pdata,
			     {website_template=>
				  ($section->{type} eq "website template")}),
		    $help,
		    $text,
		    &goto());
}

sub ManagePage {
    my $pid = shift;
    my ($page,@cdata);
    $page = $db->fetch("page",$pid);
    @cdata = $db->fetch_child("content","page",$pid);
    my $type = $page->{type};
    my $help = "\n<p>The editable content items in this $type are shown at right.  Click on one to open it.</p>\n<p>To preview the whole page, see the View menu.</p>\n";
    $help .= "<P>Page details:</P>\n".$db->report_row("page",$page);
    return &htmlbox("$type: \"$page->{title}\"",
		    &MenuBar(\@cdata),
		    $help,
		    &ShowObjects("icon",@cdata),
		    &goto($cmspath[1]));
}

sub ManageContent {
    my $id = shift;
    my ($page,$cobj,@data);
    $cobj = $db->fetch("content",$id);
    $page = $db->fetch("page",$cobj->{page_id});
    @data = $db->fetch_child("content_data","content",$id,"content_data_id");
    my $sidebar = "\n<p>Versions are shown at right.  Most recent versions are shown at top.</p>
<p>Click on the version to open it.</p>\n";
    $sidebar .= "<p>Details of \"$cobj->{name}\":</p>\n".$db->report_row("content",$cobj);
    return &htmlbox("$cobj->{name} content for $page->{filename} (\"$page->{title}\")",
		    &MenuBar(\@data),
		    $sidebar,
		    &ShowObjects("list",@data),
		    &goto(@cmspath[1..2]));
}

sub ManageContentData {
    my $id = shift;
    my $ContentData = new ExSite::ContentData(id=>$id);
    my $help = "<p>This content version is previewed at right.  It may display differently when inserted into a page with its own style rules.<p>
<P>References to unresolved content items are displayed as a checker graphic:
<img src=$config{server}{HTMLpath}/_ExSite/images/checker.png height=10 width=20 border=1>.
These can only be substituted in when this content item is displayed as part 
of a page.</P>\n";
# following only works for page.type="page"
#    $help .= "<p>(The entire page can be viewed <a href=$config{server}{CGIpath}/page.cgi?_id=$page target=preview>here</a>.)</p>\n";
    
    return &htmlbox("Version $ContentData->{cdata}{content_data_id}-$ContentData->{cdata}{ctime}",
		    &MenuBar,
		    $help,
		    #$ContentData->get_html,
		    #$ContentData->expand(mode=>"dynamic",method=>"data,url,content,page"),
		    $ContentData->preview,
		    &goto(@cmspath[1..3]));
}

sub DoAction {
    $_ = shift;
    my ($title,$text,$url);
    if (/newcontent|newdesign/) {
	$title = "New Content";
	$url = &goto(@cmspath[1..2]);
	$text .= &wizard("content","New Content",{content_id=>undef,reply=>$url});
    }
    elsif (/^(change|update)template(all)?$/) {
	my ($sel,$option,$tcheck,$type);
	$option = $1;
	$type = $2;
    	$title = "Change Template";
	$url = $type ? &goto(@cmspath[1]) : &goto(@cmspath[1..2]);
	$sel = $type ? $db->fetch('section',$section) : $db->fetch('page',$page);
	if ($type){$tcheck = ($sel->{type} !~ /template/i) ? 1 : 0;}
	else{$tcheck = ($sel->{type} =! /^page$/i) ? 1 : 0;}
	if ($tcheck){$text = ($option =~ /change/i) ? &changetemplate($type) : &updatetemplate($type);}
	else
	{
		$text = "<p>You can not change the template ";
		$text .= $type ? "for all pages" : "of a template page or a library page";
		$text .= ". :(</p>";
	}
    }
    elsif (/(change|update)body/) {
        my ($thepage,$option);
	$option = $1;
    	$title = "Change Body";
	$url = &goto(@cmspath[1..2]);
	$thepage = $db->fetch('page',$page);
	if ($thepage->{type} =~ /^page$/i){$text = ($option =~ /change/i) ? &changebody : &updatebody;}
	else{$text = "<p>You can not change the body of a template page or a library page. :(</p>";}
    }
    elsif (/editsection/) {
	$title = "Configure Website";
	$url = &goto();
	$text .= $db->make(["section"],[$section],undef,undef,{reply=>$url});
    }
    elsif (/map2page/) {
	$title = "Site Map";
	$url = &goto();
	my %s = $db->fetch("section",$section);
	$text .= "<h3>Pages for $s{title}:</h3>\n";
	$text .= MapSection("page");
    }
    elsif (/map2cms/) {
	$title = "Site Map";
	$url = &goto();
	my %s = $db->fetch("section",$section);
	$text .= "<h3>Pages for $s{title}:</h3>\n";
	$text .= MapSection("cms");
    }
    elsif (/newpage/) {
	$title = "New Page";
	$url = &goto(@cmspath[1]);
	$text .= &wizard("page","New Page",{reply=>$url});
    }
    elsif (/newtemplatepage/) {
	$title = "New Page";
	$url = &goto(@cmspath[1]);
	$text .= &wizard("website_template_page","New Page",{reply=>$url});
    }
    elsif (/newtemplate/) {
    	my $tbname = "page#_1#type";
	$title = "New Template";
	$url = &goto(@cmspath[1]);
	$text .= &wizard("template","New Template",{reply=>$url});
    }
    elsif (/newlibrary/) {
    	my $tbname = "page#_1#type";
	$title = "New Library";
	$url = &goto(@cmspath[1]);
	$text .= &wizard("library","New Library",{reply=>$url});
    }
    elsif (/addchildpage/) {
	# same as new page, but we pass in the parent id
	$title = "Add Child Page";
	$url = &goto(@cmspath[1]);
	$text .= &wizard("page","Add Child Page",{parent_id=>$page,reply=>$url});
    }
    elsif (/editpage/) {
	$title = "Configure page";
	$url = &goto(@cmspath[1..2]);
	#$text .= &wizard("page","Edit Page",{page_id=>$cmspath[2]});
	$text .= $db->make(["page"],[$page],undef,undef,{reply=>$url});
    }
    elsif (/editcontent/) {
	$title = "Configure content";
	$url = &goto(@cmspath[1..2]);
	$text .= $db->make(["content"],[$content],undef,undef,{reply=>$url});
	#$text .= &wizard("content","New Content",{content_id=>$cmspath[2]});
    }
    elsif (/content-wizard/) {
	$title = "Define Content";
	$url = &goto(@cmspath[1..3]);
	$text .= &wizard("content","Content Wizard",{content_id=>$cmspath[3]});
    }
    elsif (/contentdata-wizard/) {
	$title = "Update Content";
	$url = &goto(@cmspath[1..3]);
	$text .= &wizard("content_data","Update Content Wizard",{content_id=>$cmspath[3]});
    }
    elsif (/page-wizard/) {
	$title = "Page Wizard";
	$url = &goto(@cmspath[1..3]);
	$text .= &wizard("page","Page Wizard",{page_id=>$cmspath[2]});
    }
    elsif (/section-wizard|newsection/) {
	$title = "Website Wizard";
	$url = &goto(@cmspath[1..3]);
	$text .= &wizard("section","Website Wizard",{page_id=>$cmspath[1]});
    }
    elsif (/purge/) {
	$title = "Housecleaning";
	$url = &goto(@cmspath[1..3]);
	if ($cmspath[3]) {
	    $text .= &Purge;
	}
	else {
	    $text .= "No content to clean up!";
	}
    }
    elsif (/restore/) {
	$title = "Restore Version";
	$url = &goto(@cmspath[1..3]);
	if ($cmspath[3]) {
	    $text .= &Restore;
	}
	else {
	    $text .= "No content to promote!";
	}
    }
    elsif (/update/) {
	$title = "Update Content";
	$url = &goto(@cmspath[1..3]);
	$text .= &wizard("content_data","Update Content Wizard",{content_id=>$cmspath[3]});
    }
    else {
	$text = "<P>Unknown action.  Sorry :-(</P>\n";
    }
    return &htmlbox($title,undef,undef,$text,$url);

}

sub MapSection {
    my $opt = shift;
    my $out;
    my @pages = $db->fetch_match("page",{section_id=>$section,
					 type=>"page",
					 parent_id=>0,
					 visibility=>"visible"});
    $out .= "<P><UL CLASS=\"sitemap\">\n";
    foreach my $page (@pages) {
	$out .= &MapSection_r($page,$opt);
    }
    $out .= "</UL></P>\n";
    $out .= "<P>* Restricted-access pages.</P>\n";
    $out .= "<P>NOTE: hidden pages do not appear in the site map.</P>\n";
}

sub MapSection_r {
    # recursive mapping routine
    my ($page,$opt) = @_;
    my $pid = $page->{page_id};
    my $url;
    if ($opt eq "page") {
	# link to page
	$url = "page.cgi?_id=$pid";
    }
    else {
	# link to CMS
	$url = "CMS.cgi/$section/$pid";
    }
    my $out = "<LI><A HREF=$config{server}{CGIpath}/$url>$page->{title}</A>";
    if ($page->{access} eq "public") { $out .= " ($page->{filename})"; }
    else { $out .= "*"; }
    $out .= "</LI>\n";
    my $subpage = $db->fetch_match("page",{parent_id=>$pid,
					   type=>"page",
					   visibility=>"visible"});
    if (@$subpage > 0) {
	$out .= "<UL CLASS=\"sitemap\">\n";
	foreach my $page (@$subpage) { $out .= &MapSection_r($page,$opt); }
	$out .= "</UL>\n";
    }
    return $out;
}

sub goto {
    my $path;
    foreach my $id (@_) {
	$path .= "/$id";
	next if (! $id);
    }
    return "$CGI/CMS.cgi$path";
}

sub WhatAmI {
    my $uid = $db->my_uid;
    my $site = $section;
    if ($site) {
	my @data = $db->fetch_match("member_site_link",{section_id=>$site,
							member_id=>$uid});
	if (@data > 0) {
	    return $data[-1]->{priveleges};
	}
	else {
	    return undef;
	}
    }
    else {
	return undef;
    }
}

sub htmlbox {
    my ($title,$menubar,$sidebar,$content,$url) = @_;
    my $out = "    <TABLE CELLSPACING=0 CELLPADDING=0 BORDER=0 WIDTH=100% CLASS=framed>
      <TR>
        <TH CLASS=titlebar colspan=3><IMG SRC=$config{server}{HTMLpath}/_ExSite/images/null.gif HEIGHT=1 WIDTH=1>";
    
#    if ($url) {
#        $out .= "<A HREF=$url><IMG SRC=$config{server}{HTMLpath}/_ExSite/images/button-exit-small.gif HEIGHT=20 WIDTH=20 BORDER=0></A>";
#    }
#    else {
        $out .= "<IMG SRC=$config{server}{HTMLpath}/_ExSite/images/button-null-small.gif HEIGHT=20 WIDTH=20 BORDER=0>";
#    }

    $out .= "&nbsp;<B>$title</B></Th>
      </TR>

<!--hrule-->

      <tr><td bgcolor=#000000 colspan=3><img src=$config{server}{HTMLpath}/_ExSite/images/null.gif></td></tr>

<!--menubar-->

      <tr><td class=menubar colspan=3>$menubar</td></tr>

<!--hrule-->

      <tr><td bgcolor=#000000 colspan=3><img src=$config{server}{HTMLpath}/_ExSite/images/null.gif></td></tr>

<!--sidebar-->

      <tr><td class=sidebar WIDTH=20% valign=top>$sidebar</td>

<!--vrule-->

      <td bgcolor=#000000><img src=$config{server}{HTMLpath}/_ExSite/images/null.gif></td>

<!--webtop-->

      <td width=80% bgcolor=#FFFFFF valign=top><div class=htmlbox>
$content
      </div></td>
      </tr>
    </TABLE>\n";

}

sub MenuBar {
    my ($objects,$opt) = @_;
    # objects is a pointer to the iconified objects in the current screen
    # opt is a pointer to a datahash containing extra formatting options
    my $JS = "$config{server}{HTMLpath}/_ExSite/js";
    my $CGI = $config{server}{CGIpath};
    my $HTML = $config{server}{HTMLpath};
    my @menu;
    
    # @menu is a list of menubar-item, menu, menubar-item, menu, etc.
    # menubar-item is just a text label.  menu is a reference to an array
    # of hashes containing the menu items.  Each hash can have keys for 
    # the menu item label, the url, and (optional) the target.
    # eg. @menu=("File",[ {label=>"Exit",url=>"exit.cgi"} ] );

    if ($cdata) {
	# content data menu
	@menu = ( "Version",
		  [
		   { label=>"Delete Version", url=>"$CGI/rm.cgi?tab=content_data&id=$cdata&reply=$config{server}{CGIpath}/CMS.cgi/$section/$page/$content", perm => "administrator", },
		   { label=>"Restore Version", url=>&goto(@cmspath[1..4])."?action=restore" },
		   { label=>"Close", url=>&goto(@cmspath[1..3]) },
		   { label=>"Exit", url=>"$CGI/home.cgi" },
		   ],
		  "Update",
		  [
		   { label=>"Update", url=>&goto(@cmspath[1..4])."?action=update" },
#		   { label=>"Approve", url=>"javascript:alert('Workflow tools not available')", perm=>"administrator" },
		   # rollback == delete for now
		   { label=>"Rollback", url=>"$CGI/rm.cgi?tab=content_data&id=$cdata&reply=$config{server}{CGIpath}/CMS.cgi/$section/$page/$content/$cdata", perm => "administrator", },
		   ],
		  );
    }
    elsif ($content) {
	# content menu
	my $rollbackurl = scalar @$objects > 0 ? "$CGI/rm.cgi?tab=content_data&id=$objects->[-1]->{content_data_id}&reply=$config{server}{CGIpath}/CMS.cgi/$section/$page" : "javascript:alert('nothing to roll back!')";
	@menu = ( 
		  "Content",
		  [
		   { label=>"Update", url=>&goto(@cmspath[1..3])."?action=update" },
		   { label=>"Configure Content", url=>&goto(@cmspath[1..3])."?action=editcontent", perm=>"administrator" },
		   { label=>"Delete Content", url=>"$CGI/rm.cgi?tab=content&id=$content&reply=$config{server}{CGIpath}/CMS.cgi/$section/$page", perm=>"administrator" },
		   { label=>"Close", url=>&goto(@cmspath[1..2]) },
		   { label=>"Exit", url=>"$CGI/home.cgi" },
		   ],
		  "View",
		  [
		   { label=>"View Latest", url=>&goto(@cmspath[1..3])},
		   { label=>"View Current", url=>&goto(@cmspath[1..3])},
		   ],
		  "Update",
		  [
		   { label=>"Update", url=>&goto(@cmspath[1..3])."?action=update" },
		   { label=>"Rollback", url=>$rollbackurl, perm=>"administrator" },
		   ],
		  "Versions",
		  [
		   { label=>"Clean Up", url=>&goto(@cmspath[1..3])."?action=purge", perm=>"administrator" },
#		   { label=>"Remove Old Versions", url=>"javascript:alert('Version Clean Up not implemented')", perm=>"administrator" },
		   ],
		  );
    }
    elsif ($page) {
	# page menu
	my %page = $db->fetch("page",$page);
	my $type = ucfirst($page{type});
	# template kludge
	if ($type eq "Template") { $type = "Page"; }
	@menu = ( "$type",
		  [
		   { label=>"New Content Item", url=>&goto(@cmspath[1..2])."?action=newcontent" },
		   { label=>"New Child Page", url=>&goto(@cmspath[1..2])."?action=addchildpage", perm=>"administrator|editor" },
		   { label=>"Configure $type", url=>&goto(@cmspath[1..2])."?action=editpage", perm=>"administrator" },
		   { label=>"Publish Page", url=>"$CGI/publish.cgi?page=$page", target=>"publishwindow", perm=>"administrator|editor" },
		   { label=>"Delete $type", url=>"$CGI/rm.cgi?tab=page&id=$page&reply=$config{server}{CGIpath}/CMS.cgi/$section", perm=>"administrator", },
		   { label=>"Close", url=>&goto(@cmspath[1]) },
		   { label=>"Exit", url=>"$CGI/home.cgi" },
		   ],
		  "Template",
		  [
		   { label=>"Go to Template", url=>&goto(@cmspath[1],$page{template_id}) },
		   { label=>"Change Template",url=>&goto(@cmspath[1..2])."?action=changetemplate",perm=>"adminiaddchildpagestrator|editor|designer"},
		   { label=>"Change Body", url=>&goto(@cmspath[1..2])."?action=changebody" },
		   ],
		  "View",
		  [
		   { label=>"Preview", url=>"$CGI/page.cgi?_id=$page", target=>"preview"},
		   { label=>"Editable Preview", url=>"$CGI/page.cgi?_id=$page&_cms=1"},
		   { label=>"Non-editable Preview", url=>"$CGI/page.cgi?_id=$page&_cms=0", target=>"preview"},
		   { label=>"Edit Mode On", url=>"$CGI/page.cgi?_id=$page&_cmsmode=1"},
		   { label=>"Edit Mode Off", url=>"$CGI/page.cgi?_id=$page&_cmsmode=0"},
		   #{ label=>"Published Version", url=>"$HTML/XXX"},
		   ],
		  );
	my $update_menu = [];
	if (@$objects > 0) {
	    foreach my $c (@$objects) {
		push @$update_menu, { label=>"Update $c->{name}", url=>&goto(@cmspath[1..2],$c->{content_id})."?action=update" }
	    }
	    push @menu, ("Update", $update_menu);
	}
    }
    elsif ($section) {
	# site menu
	my %section = $db->fetch("section",$section);
	@menu = ( "Website",
		  [
		   { label=>"New Sub-Site", url=>&goto(@cmspath[1])."?action=newsection", perm=>"administrator" },
#		   { label=>"New Site Template", url=>"javascript:alert('New Site Template not implemented')", perm=>"administrator" },
		   ##### should be newtemplatepage for website templates
		   { label=>"New Page", url=>&goto(@cmspath[1])."?action=".($opt->{website_template} ? "newtemplatepage" : "newpage"), perm=>"administrator" },
		   { label=>"New Library", url=>&goto(@cmspath[1])."?action=newlibrary", perm=>"administrator" },
		   { label=>"New Template", url=>&goto(@cmspath[1])."?action=newtemplate", perm=>"administrator" },
		   { label=>"Configure Site", url=>&goto(@cmspath[1])."?action=editsection", perm=>"administrator" },
		   { label=>"Delete Site", url=>"$CGI/rm.cgi?tab=section&id=$section&reply=$config{server}{CGIpath}/CMS.cgi", perm=>"administrator"},
		   { label=>"Close", url=>&goto() },
		   { label=>"Exit", url=>"$CGI/home.cgi" },
		   ],
		  "Template",
		  [
		   { label=>"New Template", url=>&goto(@cmspath[1])."?action=newtemplate", perm=>"administrator" },
		   { label=>"Change For All Pages", url=>&goto(@cmspath[1])."?action=changetemplateall",perm=>"administrator|designer"},
		   ],
		  "Publish",
		  [
		   { label=>"Publish Site", url=>"$CGI/publish.cgi?section=$section", target=>"publishwindow", perm=>"administrator|editor" },
#		   { label=>"Un-publish Site", url=>"javascript:alert('Un-publish not implemented')", perm=>"administrator"},
		   ],
		  "View",
		  [
		   #{ label=>"Home Page", url=>"$CGI/page.cgi?_id=XXX"},
		   { label=>"Site Contents", url=>&goto(@cmspath[1])},
		   { label=>"Site Map", url=>&goto(@cmspath[1])."?action=map2cms"},
		   ],
		  );
    }
    else {
	# top menu
	@menu = ( "Websites",
		  [{ label=>"New Site", url=>&goto()."?action=newsection", perm=>"administrator" },
#		   { label=>"New Site Template", url=>"javascript:alert('New Site Template not implemented')", perm=>"administrator" },
		   { label=>"Exit", url=>"$CGI/home.cgi" }],
		  );
    }

    my $priv = &WhatAmI();
    my $out;

    # add JavaScript libraries
    $out .= "<script type='text/javascript' src='$JS/cbe_core.js'></script>
<script type='text/javascript' src='$JS/cbe_event.js'></script>
<script type='text/javascript' src='$JS/cbe_util.js'></script><!-- for debugging -->
<script type='text/javascript' src='$JS/cbe_debug.js'></script><!-- for debugging -->
<script type='text/javascript' src='$JS/menu_exsite.js'></script>

<script type='text/javascript'><!--
var mark1, menu1, menu2, menu3;
function windowOnload() {
  mark1 = cbeGetElementById('menuMarker1').cbe;
  menu1 = new cbeMenu(
    true,                       // horizontal,
//    (document.cbe.width()-(3*75))/2, mark1.pageY(), 3, // centered
    mark1.pageX(), mark1.pageY(), 3, // menuX, menuY, menuZ (left-justified)
    [75,75,75], 20, 8,          // lblWidthsArray, lblHeight, lblspacing
    [140,170,155],              // boxWidthsArray,
    '#000099', '#000000',       // activeColor, inactiveColor,
    '#FFFFFF', '#9999AA',       // activeBkgnd, inactiveBkgnd
    '#9999AA'                   // boxBkgnd
  );
  window.cbe.addEventListener(\"resize\", resizeListener, false);
}

--></script>\n";

    # now build the HTML for the menubar

    $out .= "<a name=\"topofpage\"></a>
 <TABLE WIDTH=100% CELLSPACING=0 CELLPADDING=0 BORDER=0> 
  <TR>
   <TD COLSPAN=4><IMG SRC=$config{server}{HTMLpath}/_ExSite/images/null.gif HEIGHT=1 WIDTH=1></TD>
  </TR>
   <TR>
   <TD><IMG SRC=$config{server}{HTMLpath}/_ExSite/images/null.gif HEIGHT=1 WIDTH=1></TD>
   <TD>
     &nbsp;&nbsp;
   </TD>
   <TD WIDTH=100%>
    <div id='menuMarker1' class='menuMarker'>&nbsp;</div>
   </TD>
   <TD><IMG SRC=$config{server}{HTMLpath}/_ExSite/images/null.gif HEIGHT=1 WIDTH=1></TD>
  </TR>
  <TR>
   <TD COLSPAN=4><IMG SRC=$config{server}{HTMLpath}/_ExSite/images/null.gif HEIGHT=1 WIDTH=1></TD>
  </TR>
 </TABLE>\n";

    # now create the menus...

    my $priv = &WhatAmI();
    my $imenu = 0;
    while (@menu) {
	$imenu++;
	my $menutitle = shift @menu;
	my $menu = shift @menu;

	$out .= "<div id='label1$imenu' class='mLabel'>$menutitle</div>
<div id='box1$imenu' class='mBox'>\n";
	foreach my $item (@$menu) {
	    next if (exists $item->{priv} && $priv !~ /$item->{priv}/);
	    $out .= "<a class=\"m\" href=\"$item->{url}\"";
	    if (exists $item->{target}) { $out .= " target=\"$item->{target}\""; }
	    $out .= ">$item->{label}</a>\n";
	}
	$out .= "</div>\n";
    }

    return $out;

}

#=======================================================================
# clean out old content
#=======================================================================

sub Purge {
    my $out;
    my $first = 1;
    foreach my $key (keys %formdata) {
	if ($key =~ /^\d+$/) {
	    $first = 0;
	    $db->delete_key("content_data",$key);
	}
    }
    my @cdata = $db->fetch_child("content_data","content",$content,"content_data_id");
    my $url = &goto(@cmspath[1..3]);
    $out .= "<H1>Housecleaning</h1>\n<p>This form allows you to remove old content versions that no longer need to be archived.  Select which versions to delete.</p>\n<p><b>Delete:</b></p>
<p><form action=\"${url}?action=purge\" method=\"post\">\n";
    for (my $ic = $#cdata; $ic >= 0; $ic--) {
	$out .= "<input type=checkbox name=\"$cdata[$ic]->{content_data_id}\"";
	if ($first && $ic != $#cdata) {
	    # select all non-current versions by default
	    $out .= " checked";
	}
	##### should link the version to a preview
	$out .= "> $cdata[$ic]->{note}<br>\n";
    }
    $out .= "</p>\n<input type=\"submit\" value=\"Delete\"> <input type=reset>\n";
    return $out;
}

#=======================================================================
# restore old content
#=======================================================================

sub Restore {
    my %cdata = $db->fetch("content_data",$cdata);
    my $message = "Restored content_data $cdata";
    $cdata{note} = $message;
    delete $cdata{content_data_id};
    $db->insert("content_data",\%cdata);
    return "<h1>$message</h1>";
}

#=======================================================================
# webtop and icon display
#
#=======================================================================

sub ShowObjects {
    my $format = shift;  # "icon" or "list"
    # remaining args are the data objects
    my $icon = ($format eq "icon");

    my $out = "<table cellspacing=20><tr>\n";
    my $nicon;
    if ($icon) {
	foreach my $obj (@_) {
	    $nicon++;
	    my ($iconname,$label,$description,$url) = &objtype($obj);
	    $out .= "<td class=icon><a href=\"$url\" class=icon title=\"$description\"><img src=$config{server}{HTMLpath}/_Modules/CMS/${iconname}.gif height=48 width=48><br>$label</a></td>\n"; 
	    if ($nicon % 6 == 0) { $out .= "</tr><tr>\n"; }
	}
	if (@cmspath > 1) {
	    $nicon++;
	    my $url = &goto(@cmspath[1..$#cmspath-1]);
	    $out .= "<td class=icon><a href=\"$url\" class=icon title=\"back\"><img src=$config{server}{HTMLpath}/_Modules/CMS/back.gif height=48 width=48><br>[ up ]</a></td>\n"; 
	}
    }
    else {
	# list in inverse order
	while (@_) {
	    my $obj = pop @_;
	    my ($iconname,$label,$description,$url) = &objtype($obj);
	    $out .= "<td class=icon><a href=\"$url\" class=icon title=\"$description\"><img src=$config{server}{HTMLpath}/_Modules/CMS/${iconname}.gif height=48 width=48></a></td><td><a href=\"$url\" class=icon title=\"$description\">$label</a></td>\n</tr></tr>\n"; 
	}
	if (@cmspath > 1) {
	    $nicon++;
	    my $url = &goto(@cmspath[1..$#cmspath-1]);
	    $out .= "<td class=icon><a href=\"$url\" class=icon title=\"back\"><img src=$config{server}{HTMLpath}/_Modules/CMS/back.gif height=48 width=48></td><td><a href=\"$url\" class=icon title=\"back\">[back]</a></td>\n</tr><tr>\n"; 
	}
    }
    $out .= "</tr></table>\n";
    return $out;
}

# return the object type
# site, sitetemplate, page, template, library, econtent, dcontent, data
sub objtype {
    my $obj = shift;
    if (exists $obj->{section_id}) {
	if (exists $obj->{publish_to}) {
	    # a site/section
	    return (($obj->{type} =~ /template/ ? "sitetemplate" : "site"),
		    $obj->{title},
		    $obj->{description},
		    &goto($obj->{section_id}));
	}
	else {
	    # a page
	    return ($obj->{type},
		    $obj->{filename},
		    $obj->{description},
		    &goto($obj->{section_id}, $obj->{page_id}));
	}
    }
    elsif (exists $obj->{page_id}) {
	# content object
	return (($obj->{type} eq "editorial" ? "econtent" : "dcontent"),
		$obj->{name},
		"$obj->{allow_mime}; $obj->{description}",
		&goto($cmspath[1],$obj->{page_id},$obj->{content_id}));
    }
    else {
	# content data
	return ("data",
		"Updated ".$db->show_data("content_data","ctime",$obj->{ctime})."; $obj->{note}",
		$obj->{mime_type},
		&goto($cmspath[1],$cmspath[2],$obj->{content_id},$obj->{content_data_id}));
    }
}


#=======================================================================
# wizards
#
# Wizards are multi-screen forms that save their inputs in hidden fields
# until the last screen is submitted.
#=======================================================================

sub wizard {
    my ($type,$title,$init) = @_;

    my $wiz = new ExSite::Wizard(title=>$title);

    if ($type eq "content_data") {
	#$wiz->add_step("Update Method",\&wiz_cdata_page1,\&do_wiz_cdata_page1);
	$wiz->add_step("Enter New Content",\&wiz_cdata_page2,\&do_wiz_cdata_page2);
	$wiz->add_step("Content Type and Comment",\&wiz_cdata_page3,\&do_wiz_cdata_page3);
    }
    elsif ($type eq "content") {
	$wiz->add_step("Content Parameters",\&wiz_content_page1,\&do_wiz_content_page1);
	#$wiz->add_step("Update Method",\&wiz_cdata_page1,\&do_wiz_cdata_page1);
	$wiz->add_step("Enter New Content",\&wiz_cdata_page2,\&do_wiz_cdata_page2);
	$wiz->add_step("Content Type and Comment",\&wiz_cdata_page3,\&do_wiz_cdata_page3);
    }
    elsif ($type eq "page") {
	$wiz->add_step("Details",\&wiz_page_page1,\&do_wiz_page_page1);
	$wiz->add_step("Template",\&wiz_page_page2,\&do_wiz_page_page2);
	$wiz->add_step("Body",\&wiz_page_page3,\&do_wiz_page_page3);
    }
    elsif ($type eq "template") {
	$wiz->add_step("Details",\&wiz_template_page1,\&do_wiz_template_page1);
	$wiz->add_step("Parent Template",\&wiz_template_page2,\&do_wiz_template_page2);
    }
    elsif ($type eq "website_template_page") {
	$wiz->add_step("Details",\&wiz_page_page1,\&do_wiz_page_page1);
	# no template selection in this case
	$wiz->add_step("Body",\&wiz_templatepage_page2,\&do_wiz_templatepage_page2);
    }
    elsif ($type eq "library") {
	$wiz->add_step("Details",\&wiz_library_page1,\&do_wiz_library_page1);
    }
    elsif ($type eq "section") {
	$wiz->add_step("Details",\&wiz_section_page1,\&do_wiz_section_page1);
	$wiz->add_step("Location",\&wiz_section_page2,\&do_wiz_section_page2);
	$wiz->add_step("Pages",\&wiz_section_page3,\&do_wiz_section_page3);
	$wiz->add_step("Webmaster",\&wiz_section_page4,\&do_wiz_section_page4);
    }

    return $wiz->go($init);

}

########################
# content_data wizard
########################

sub wiz_cdata_page1 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{content_id};
    my ($out, %content, @cdata, $selected);
    # set reply page on first screen
    if (! exists $in->{reply}) {
	my $reply = $ENV{HTTP_REFERER} || 
	    "$config{server}{CGIpath}/CMS.cgi$ENV{PATH_INFO}";
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }
    if ($id =~ /\d+/) {
	%content = $db->fetch("content",$id);
	@cdata = $db->fetch_child("content_data","content",$id,"content_data_id");
    }
    if (@cdata == 0) {
	$cdata[0] = { content_data_id=>"_2", mime_type => "text/html" };  # default mime type
    }
    my $cid = $cdata[-1]{content_data_id};
    my $cid = $cdata[-1]{content_data_id};
    $out .= $db->input_html(type=>"hidden",name=>"content_data#_2#content_id", value=>$id);
    my $name = $content{name} || $in->{"content#${id}#name"};
    $out .= "<p>This wizard will guide you through the steps of updating \"$name\".</p>\n";
    if (! exists $in->{content_id}) {
	# we've not come from the content wizard, so we'll need to include
	# a reply screen
	$out .= "<input type=hidden name=\"reply\" value=\"$in->{reply}\">\n";
    }
    $out .= "<input type=hidden name=\"content_data_id\" value=\"$cid\">\n"; 
#    if ($id & @cdata > 0) {
#	######### should in-line the content into the same screen
#	$out .= "<p><a href=javascript:popup_link(\"$CGI/viewcontent.cgi?id=$cid\")>View the latest version of \"$content{name}\" here</a>.</p>\n";
#	$out .= "<script LANGUAGE=javascript>\nfunction popup_link(url) {\n  window.open(url,\"\",\"width=400,height=400,resizable,scrollbars=1,menubar=0\")\n}\n</SCRIPT>\n";
#    }
    $out .= "<p>Please select how you will be updating this content:</p>\n";
    if ($cdata[-1]->{mime_type} =~ /^text\/html$/o) {
	$out .= "<p><input type=radio name=\"update_method\" value=\"html\" checked> Edit using WYSIWYG HTML editor</p>\n";
	$selected = 1;
    }
    else {
	$out .= "<p><input type=radio name=\"update_method\" value=\"newhtml\"> Replace with HTML, using WYSIWYG editor</p>\n";
    }
    if ($cdata[-1]->{mime_type} =~ /^text/o) {
	$out .= "<p><input type=radio name=\"update_method\" value=\"edit\"";
	if (! $selected) { $out .= " checked"; $selected = 1; }
	$out .= "> Edit using a simple text editor</p>\n";
    }
    else {
	$out .= "<p><input type=radio name=\"update_method\" value=\"newedit\"> Replace with text, using a simple text editor</p>\n";
    }
    $out .= "<p><input type=radio name=\"update_method\" value=\"upload\"";
    if (! $selected) { $out .= " checked"; $selected = 1; }
    $out .= "> Upload new version in a file</p>\n";
    $out .= "<p><input type=radio name=\"update_method\" value=\"file\"> Point to a file already in place on the server</p>\n";
    $out .= "<p><input type=radio name=\"update_method\" value=\"url\"> Point to a 3rd-party resource from another website</p>\n";
    return $out;
}

sub wiz_cdata_page2 {
    my $this = shift;
    my $in = shift;
    my $id;
    my ($out, %content, @cdata, $selected);
    
    # set reply page on first screen
    if (! exists $in->{reply}) {
	my $reply;
	if ($ENV{HTTP_REFERER} =~ /page\.cgi/) {
	    $reply = $ENV{HTTP_REFERER};
	}
	else {
	    $reply = &goto(@cmspath[1..3]);
	}
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }

    if (exists $in->{'content_data#_2#content_id'}) {
	#$out .= "resubmit! removing content_data values<br>\n";
	# this is a resubmit of screen 1 - delete all record data so the form
	# can be regenerated fresh (with a new update method, though)
#### warning: this code causes segmentation faults (!?) 
#	foreach my $key (%$in) {
#	    if ($key =~ /$config{form}{sepchar}/) {
#		delete $in->{$key};
#	    }
#	}
#### we'll use the brute-force method instead...
	$id = $in->{'content_data#_2#content_id'};
	delete $in->{'content_data#_2#content_id'};
	delete $in->{'content_data#_2#data#html'};
	delete $in->{'content_data#_2#fdata#filename'};
    }
    else {
	#$out .= "no resubmit - adding content_id values<br>\n";
	# get the content id from the init data
	$id = $in->{content_id};
    }
    $out .= $db->input_html(type=>"hidden",name=>"content_data#_2#content_id", value=>$id);

    if ($id =~ /\d+/) {
	%content = $db->fetch("content",$id);
	@cdata = $db->fetch_child("content_data","content",$id,"content_data_id");
    }
    if (@cdata == 0) {
	# set an assumed mime type, depending on content type
	if ($in->{'content#_1#type'} eq "design") {
	    # also check for content#_1#allow_mime#_set_N = "image"
	    $cdata[0] = { content_data_id=>"_2", mime_type => "image/gif" };
	}
	else {
	    $cdata[0] = { content_data_id=>"_2", mime_type => "text/html" };
	}
    }
    my $cid = $cdata[-1]{content_data_id};
    my $name = $content{name} || $in->{"content#${id}#name"};
    $out .= "<p>This wizard will guide you through the steps of updating \"$name\".</p>\n";
#    if (! exists $in->{content_id}) {
#	########## will always exist due to line 3
#	# we've not come from the content wizard, so we'll need to include
#	# a reply screen
#	$out .= "<input type=hidden name=\"reply\" value=\"$in->{reply}\">\n";
#    }
    if (! exists $in->{content_data_id}) {
	$out .= "<input type=hidden name=\"content_data_id\" value=\"$cid\">\n"; 
    }

    my $update_method = $in->{update_method};
    if ($update_method =~ /$config{form}{multi_sep}/) {
	# we now have multiple update methods (due to reselecting the 
	# update method several times).  Delete all but the most recent.
	my @update_methods = split /$config{form}{multi_sep}/,$update_method;
	$update_method = @update_methods[-1];
	#$in->{update_method} = $update_method;
    }

    if (! $update_method) {
	if ($cdata[-1]->{mime_type} =~ /^text\/html$/o) {
	    $update_method = "html";
	}
	elsif ($cdata[-1]->{mime_type} =~ /^text/o) {
	    $update_method = "edit";
	}
	else {
	    $update_method = "upload";
	}
    }

    $out .= "<script>
function formsubmit(){
    document.forms[0].screen.value--;
    document.forms[0].submit();
}
</script>\n";
    $out .= "<p>Update method: \n<select name=\"update_method\" onchange=\"formsubmit()\">\n";
    $out .= "<option value=\"html\"";
    if ($update_method =~ /html/) { $out .= " selected"; }
    $out .= ">Edit using WYSIWYG HTML editor</option>\n";
    $out .= "<option value=\"edit\"";
    if ($update_method =~ /edit/) { $out .= " selected"; }
    $out .= ">Edit using a simple text editor</option>\n";
    $out .= "<option value=\"upload\"";
    if ($update_method eq "upload" || ! $update_method) { 
	$out .= " selected"; 
    }
    $out .= ">Upload new version in a file</option>\n";
    $out .= "<option value=\"file\"";
    if ($update_method eq "file") { $out .= " selected"; }
    $out .= ">Point to file on server</option>\n";
    $out .= "<option value=\"url\"";
    if ($update_method eq "url") { $out .= " selected"; }
    $out .= ">Point to URL</option>\n";
    $out .= "</select>\n";
    $out .= "<br><img src=\"$config{server}{HTMLpath}/_ExSite/images/caution20.gif\" height=\"20\" width=\"20\" border=\"0\" align=\"middle\"> Changing the update method will revert to the most recent saved content.\n";

    # step 2
    my $textdata;
    if ($update_method !~ /^new/ && $cid !~ /^_/) {
	my $c = new ExSite::ContentData(content_data=>$cdata[-1]);
	if ($c->is_text) {
	    $textdata = $c->get_raw;
	}
    }
    if ($update_method =~ /html$/ && $db->use_wysiwyg) {
	$out .= "<p>You may enter your content changes directly into this WYSIWYG editor:</p>\n";
	$out .= $db->input_htmleditor(name=>"content_data#_2#data",
				      table=>"content_data",
				      record=>$cid,
				      page=>$page,
				      column=>"data",
				      value=>"$textdata",
				      editor=>{
					  op_image=>1,
					  op_link=>1,
					  op_table=>1,
					  op_dyncontent=>1,
				      });
	$out .= "<p>Note: unresolved content is represented using the missing graphic image(<img src=$config{server}{HTMLpath}/_ExSite/images/checker.png HEIGHT=10 WIDTH=20 border=1>).  This missing content can only be shown when this content is displayed as part of a complete page.</p>\n";
    }
    elsif ($update_method =~ /^upload$/) {
	$out .= "<p>Click on browse to select a file to upload:</p>\n";
	$out .= "<p><input type=file size=60 name=\"content_data#_2#fdata#filename\"></p>\n";
	$this->{db}->{enctype} = "multipart/form-data";
	######################## need a multipart-encoded form!
	########## should be done by Form
    }
    elsif ($update_method =~ /^file$/) {
	$out .= "<p>Enter the full path to the file on the server:</p>\n";
	$out .= "<p><input type=text size=80 name=\"content_data#_2#file\"></p>\n";
	$out .= "<p>WARNING: content in external files is outside of ExSite's revision control system.  The file may be deleted or modified without ExSite's knowledge, which could affect the correct rendering of your page.  To bring this file into ExSite's revision control, use the \"Upload new version in a file\" option, instead. (You may first have to copy the file from the server to your computer.)</p>\n";
    }
    elsif ($update_method =~ /^url$/) {
	$out .= "<p>Enter the full URL to the web-based content:</p>\n";
	$out .= "<p><input type=text size=80 name=\"content_data#_2#url\"></p>\n";
	$out .= "<p>WARNING: content on remote websites is beyond the control of ExSite.  If the remote webmaster changes or removes this content, your page may not render correctly.</p>\n";
	$out .= "<p>Also note that remote content may be covered by copyright.  Although technically you are not copying this content, you may end up giving the appearance of having done so.  Please ensure that you will not be offending the owners of this content by linking to it in this way.</p>\n";
    }
    else {
	# default to text area
	$out .= "<p>You may enter your content changes directly into this text editing area:</p>\n";
	$out .= "<p><textarea cols=80 rows=20 wrap=virtual name=\"content_data#_2#data\">\n";
	$out .= $textdata;
	$out .= "</textarea></p>\n";
    }
    return $out;
}

sub wiz_cdata_page3 {
    my $this = shift;
    # screen's html in a hidden input
    my $in = shift;
    my $id = $in->{content_data_id};
    my $out;
    
    # 1) mime-type

    my $mimetype;
    if ($in->{update_method} =~ /html/) {
	$mimetype = "text/html";
    }
    elsif ($in->{update_method} =~ /edit/) {
	# could be html, plain text, or css
	#$out .= &ShowHash($in);
	if ($in->{"content_data#_2#data"} =~ /<(!--|[\/a-zA-Z])/) {
	    $mimetype = "text/html";
	}
	elsif ($in->{"content_data#_2#data"} =~ /\/\*|[\w-]+:\s*[\w\#]+\;|^\w*\.\w+\s*{/) {
	    $mimetype = "text/css";
	}
        else {
	    $mimetype = "text/plain";
	}
    }
    else {
	my $file;
	if ($in->{update_method} =~ /upload/) {
	    ($file,undef) = split(/\#/,$in->{"content_data#_2#fdata"});
	}
	else {
	    $file = $in->{"content_data#${id}#file"} || $in->{"content_data#_2#url"};
	}
	# guess based on file name
	$mimetype = &MimeType($file);
    }

    # above mimetype is just our best guess.  Show options, so that it
    # can be over-ridden.

    $out .= "<ul>\n";
    $out .= "<li><p>ExSite thinks this content is of type ";
    $out .= $db->input_exsite(datatype=>"list:mime", name=>"content_data#_2#mime_type", 
			      value=>$mimetype);
    $out .= "</p>\n<p>(If ExSite has guessed incorrectly, you can set the content type manually.)</p></li>\n";

    # 2) attributes

    if ($mimetype =~ /image/) {
	$out .= "<li><p>Include any special image attributes you would like applied to this image (optional):</p>\n";
	# FIXME: automatically add the height and width ?
	$out .= $db->input_exsite(datatype=>"string", size=>60, name=>"content_data#_2#attributes");
	$out .= "</li>\n";
    }

    # 3) revision control comment

    $out .= "<li><p>For the purposes of auditing, please include a note describing the changes you made:</p>\n";
    $out .= $db->input_exsite(datatype=>"text",size=>100,name=>"content_data#_2#note");
    $out .= "</li></ul>\n";

    $out .= "<p>Note that content updates may require approval before becoming visible on public pages.</p>\n";
	
    if ($mimetype =~ /^text/) {
	$out .= "<p><img src=\"$config{server}{HTMLpath}/_ExSite/images/caution20.gif\" height=\"20\" width=\"20\" border=\"0\" align=\"middle\"> Hitting \"Back\" may revert to the last saved version of your content.</p>\n";
    }

    return $out;
}

sub do_wiz_cdata_page1 {
    my $this = shift;
    my $in = shift;
    my @err;
    if (! $in->{update_method}) {
	push @err, "Please specify which update method you want to use.";
    }
    return ();
}

sub do_wiz_cdata_page2 {
    my $this = shift;
    my $in = shift;
    my @err = &do_wiz_cdata_page1($this,$in);
    if (! ($in->{"content_data#_2#data"} || $in->{"content_data#_2#fdata"} || 
	   $in->{"content_data#_2#file"} || $in->{"content_data#_2#url"})) {
	push @err, "Please provide some content!";
    }
    # set up file data
    if ($in->{"content_data#_2#fdata#filename"}) {
	$in->{"content_data#_2#fdata"} = $this->{db}{form}{content_data}{_2}{fdata};
	delete $in->{"content_data#_2#fdata#filename"};
    }
    return ();
}

sub do_wiz_cdata_page3 {
    my $this = shift;
    my $in = shift;
    $this->unescape_input($in);
    my $id = $in->{content_data_id};
    my @err;
    if ($id) {
	@err = &do_wiz_cdata_page2($this,$in);
	if (! $in->{"content_data#_2#mime_type"}) {
	    push @err, "Please ensure that a content type is selected.";
	}
	if (! $in->{"content_data#_2#note"}) {
	    push @err, "Please provide a note explaining this content update.";
	}
    }
    return @err if (@err > 0);

    # fix reply value (KLUDGE!)  ############

    if ($db->{data}{reply} =~ /;/) {
	my @replies = split /;/, $db->{data}{reply};
	$db->{data}{reply} = $replies[0];
	push @err, "reply = $db->{data}{reply}";
    }
    return @err if (@err > 0);

    # if we get here, we're good to go

#    # 1) update/insert the content item
#
#    my $tmpform = $db->{form};  # save parsed form
#    delete $db->{form}->{content_data};
#    if ($in->{content_id}) {
#	$db->{action} = "update";
#    }
#    else {
#	$db->{action} = "insert";
#    }
#    my $do_out = $db->do;
#    if (@{$db->{errors}} > 0 || $id) {  # exit now if just content object
#	return ($do_out);
#    }
#    my $content_id = $in->{content_id} || $db->{last_insert_id};

    # 2) insert the new content data

#    $db->{form} = $tmpform;  # restore parsed form
#    delete $db->{form}->{content};
    # now add the content id to the content data record
#    $db->{form}->{_}->{content_data}->{content_id} = $content_id;
    $db->{action} = "insert";
    $db->{form}->{content_data}->{_2}->{data} =~ s/%7B/{/g;
    $db->{form}->{content_data}->{_2}->{data} =~ s/%7D/}/g;
    my $do_out = $db->do;
    if ($db->errorcheck) {
	return ($do_out);
    }
    if (! $db->{last_insert_id}) {
	return ("Content update failed!");
    }
    return ();
}

########################
# content wizard
########################

sub wiz_content_page1 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{content_id};
    my %content;
    if ($id) {
	%content = $db->fetch("content",$id);
	$db->{action} = "update";
    }
    else {
	$id = "_1";
	%content = (content_id => $id );
	$db->{action} = "insert";
    }
    my $out = "<p>A content item is like a variable that can contain website content.  
It has a name, which used to reference the content, and 
values (or versions), which can change over time.</p>
<p>On this first screen, you define the content's name, category, and the 
types of content it can legally hold.</p>\n";

    # set reply page on first screen
    if (! exists $in->{reply}) {
	my $reply = $ENV{HTTP_REFERER} || 
	    "$config{server}{CGIpath}/CMS.cgi$ENV{PATH_INFO}";
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }

    $out .= "<p>Name of content item:<br>\n";
    $out .= $db->input_column(table=>"content",column=>"name",size=>20,
			      record=>$id,data=>\%content);
    $out .= "</p>\n<p>A brief description of this content item (optional):<br>\n";
    $out .= $db->input_column(table=>"content",column=>"description",
			      record=>$id,data=>\%content);
    $out .= "</p>\n<p>Content category (determines who is allowed to update this content item):<br>\n";
    $out .= $db->input_column(table=>"content",column=>"type",
			      record=>$id,data=>\%content);
    $out .= "</p>\n<p>The allowed types of content for this content item (uncheck everything to allow all content types):<br>\n";
    $out .= $db->input_column(table=>"content",column=>"allow_mime",
			      record=>$id,data=>\%content);

    $out .= "\n<br><br><P>".$db->input_html(type=>"checkbox",name=>"skip_update")." Skip remaining content-update steps.  Select this if not actually creating/modifying this content at this time.</p>\n";
    $out .= $db->input_html(type=>"hidden", name=>"reply", value=>$in->{reply});
    $out .= $db->input_html(type=>"hidden", name=>"content_id", value=>$id);
    $out .= $db->input_html(type=>"hidden", name=>"content#${id}#page_id", value=>($content{page_id}||$page));
			    
    return $out;
}

sub do_wiz_content_page1 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{content_id};
    my @err;
    if (! $in->{"content#${id}#name"}) {
	push @err, "Please specify a name for this content item.";
    }
    if (! $in->{"content#${id}#type"}) {
	push @err, "Please specify a category for this content item.";
    }
    return @err if (@err > 0);
    if ($in->{skip_update}) {
	# manually reset to last step
	$this->{current} = 3;
	#$in->{screen} = 3;

	#return &do_wiz_cdata_page3($this,$in);

	$db->{action} = "insert";
	my $do_out = $db->do;
	if ($db->errorcheck) {
	    return ($do_out);
	}
	if (! $db->{last_insert_id}) {
	    return ("Content update failed!");
	}
    }
    return ();
}

########################
# page wizard
########################

sub wiz_page_page1 {
    my $this = shift;
    my $in = shift;
    my %page;
    my $id = "_1";
    $db->{action} = "insert";
    my $out;
    $out .= $db->input_html(name=>"page#${id}#type",
			    value=>"page",
			    type=>"hidden");
    if ($in->{reply}) {
	$out .= $db->input_html(name=>"reply",
				type=>"hidden",
				value=>"$in->{reply}");
    }
    
    $out .= "<h4>Page Description</h4>\n<blockquote>\n";
    $out .= $db->input_html(name=>"page_id",type=>"hidden",value=>$id);
    $out .= $db->input_html(name=>"page#${id}#section_id",
			    type=>"hidden",
			    value=>$section);
    $out .= "<p>Title:<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"title",
			      record=>$id);
    $out .= "<p>Description:<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"description",
			      record=>$id);
    $out .= "<p>Status:<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"status",
			      record=>$id);
    $out .= "</p>\n</blockquote>\n";
    $out .= "<h4>Page Viewing Parameters</h4>\n<blockquote>\n";
    $out .= "<p>Accessibility:<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"access",
			      record=>$id);
    $out .= "</p>\n<p>Is this page static or dynamic?<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"publish_method",
			      record=>$id);
    $out .= "</p>\n</blockquote>\n";

    $out .= "<h4>Page Positioning</h4>\n<blockquote>\n";
    $out .= "<p>Is this page visible in menus and site maps?<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"visibility",
			      record=>$id);
    $out .= "</p>\n<p>Specify the parent page (sets the position in menus and site maps):<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"parent_id",
			      record=>$id, 
			      value=>$in->{parent_id},
			      fmatch=>{section_id=>$section, type=>"page"});
    $out .= "</p>\n<p>Menu Rank (typically 0-9):<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"rank",
			      record=>$id);
    $out .= "</p>\n<p>Menu label for this page:<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"label",
			      record=>$id);
    $out .= "</p>\n</blockquote>\n";
    $out .= "<h4>Meta-information</h4>\n<blockquote>\n";
    $out .= "</p>\n<p>Keywords:<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"keywords",
			      record=>$id);
    $out .= "</p>\n";

    $out .= "</p>\n</blockquote>\n<h4>Publishing info</h4>\n<blockquote>\n<p>File name (must end in \".htm\", \".html\", or \".php\"):<br>\n";
    $out .= $db->input_column(table=>"page",
			      column=>"filename",
			      record=>$id);
    $out .= "</p>\n</blockquote>\n";
    return $out;
}

sub do_wiz_page_page1 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};
    my @err;
    if (! $in->{"page#${id}#type"}) {
	push @err, "No page type specified!";
    }
    if (! $in->{"page#${id}#title"}) {
	push @err, "Please specify a page title.";
    }
    if (! $in->{"page#${id}#status"}) {
	push @err, "Please specify a page status.";
    }
    if (! $in->{"page#${id}#access"}) {
	push @err, "Please specify the page accessibility.";
    }
    if ($in->{"page#${id}#access"} ne "public" && 
	$in->{"page#${id}#publish_method"} eq "static") {
	push @err, "Only public pages can be static;  please select a different accessibility or static/dynamic setting.";
    }
    if (! $in->{"page#${id}#visibility"}) {
	push @err, "Please specify the page visibility.";
    }
    if ($in->{"page#${id}#visibility"} ne "hidden" && ! $in->{"page#${id}#label"}) {
	push @err, "Please specify a menu label.";
    }
    if ($in->{"page#${id}#filename"} !~ /^[\w-]+\.(html?|php)$/) {
	push @err, "Invalid filename -- please use the characters A-Z, a-z, 0-9, -, and _ only.  The filename must end in \".htm\", \".html\", or \".php\".";
    }
    return @err;
}

sub wiz_page_page2 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %page;
    my $id = $in->{page_id};

    $out .= "<p>Select the template that will define this page's layout, 
graphics, and style.  (Click on the preview links to view the templates.)</p>\n";

    $out .= &list_template("page#_1#template_id#key");

    $out .= "</p>\n<p>If you copy the layout (the \"page\" object) from 
the template, you will be able to make layout customizations for this 
page only.  However, you will no longer see updates to the layout of the 
template.  Only select this option if you are certain that this page
requires customizations to the layout.</p>
<blockquote><p>\n".
        $db->input_html(type=>"checkbox",name=>"copy_layout",selected=>0).
        " Copy Page Layout</p></blockquote>\n";

    return $out;
}

sub do_wiz_page_page2 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};
    my @err = &do_wiz_page_page1($this,$in);
    if ($in->{skip_template} || ! $in->{"page#${id}#template_id#key"}) {
	push @err, "Please select a template.";
    }
    return @err;
}

sub wiz_page_page3 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %page;
    my $pdata;
    my $id = $in->{page_id};

    ########### compare to custom version, if it exists

    %page = $db->fetch("page",$in->{"page#${id}#template_id#key"});
    my $p = new ExSite::Page(page=>\%page);
    $p->set_context;
    my $pdata = $p->get_html;

    if ($pdata =~ /\<\!--\&content\(body\)--\>/) {
	# this page has a conventional body
	# list all body templates available
	my (@liblist,$libs,$templates,$sql);
        $libs = $db->fetch_match('page',{type=>"library",section_id=>$section});
	$out .= "<p>This page appears to have a conventional layout, with a \"body\" area for holding page-specific content.</p>\n";
	$out .= "<input type=hidden name=content#_2#name value=\"body\">
<input type=hidden name=content#_2#type value=\"editorial\">
<input type=hidden name=content#_2#page_id value=\"$id\">\n";
	if (@$libs > 0) {
	    $out .= "<p>You can optionally select a pre-defined body for this 
page, from the following list of available body templates.</p>\n<p>";
	    foreach my $l (@$libs) {
		push(@liblist, $l->{page_id});
	    }
	    ######### FIXME: re-write this to avoid a custom query
	    $sql = "select * from content where name='body' and page_id in (".join(',',@liblist).")";
	    $templates = $db->custom_query($sql);
	    #$out .= "<input type=\"hidden\" name=\"content_data#_3#content_id\" value=\"_2\">\n";
	    $out .= "<input type=\"radio\" name=\"body_id\" value=\"0\" checked> normal body, fully editable (will be blank to start with)<br>\n";
	    foreach my $t (@$templates) {
		$out .= "<input type=\"radio\" name=\"body_id\" value=\"$t->{content_id}\">$t->{description}<br>\n";
	    }
	}
    }
    elsif (! $pdata) {
	# this page has no framework 
	$out .= "<IMG SRC=$config{server}->{HTMLpath}/_ExSite/images/caution.png HEIGHT=44 WIDTH=53 ALIGN=left>
<p>This page has no layout defined at this time.  There is no further
pre-configuration ExSite can do on this page -- please proceed
by clicking on FINISH.</p>
<p>Later, you will need to manually add the basic layout framework for this page.
To do this, create a content item named \"page\", and fill it with an
ExSite template (HTML with embedded ExSite templating tags).</p>\n";
    }
    else {
	# this page has an irregular layout, with no body
	$out .= "<IMG SRC=$config{server}->{HTMLpath}/_ExSite/images/caution.png HEIGHT=44 WIDTH=53 ALIGN=left>
<p>This page appears to have an irregular layout, with no explicit 
\"body\" area for holding page-specific content.  This is not an error,
since some pages have special layout requirements that cannot be met by
a conventional layout.  However, there is
no further pre-configuration ExSite can do on this page -- please proceed
by clicking on FINISH. Customization of this page can be performed
using the regular content-management tools, once the page is created.</p>\n";
    }

    $out .= "<br><br><br>\n";

    return $out;
}

sub do_wiz_page_page3 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};

    my @err = &do_wiz_page_page2($this,$in);
    if (@err > 0) { return @err; }
    
    # now, perform all the necessary DB mods

    # copy layout

    if ($in->{copy_layout}) {  ## && $in->{'page#_1#template_id#key'}
	&wiz_copy_page_layout($this,$in);
    }
    
    # copy selected page body

    if ($in->{body_id}) {
	# substitute the selected body into the form data
	my $c_data = $db->fetch_child("content_data","content",$in->{"body_id"},"content_data_id");
	$c_data = $c_data->[-1];
	$cdata->{note} = "copied from content_data $cdata->{content_data_id}";
	$cdata->{content_id} = "_2";
	delete $cdata->{content_data_id};
	delete $cdata->{ctime};
	$db->{'form'}{'content_data'} = { '_3' => $c_data };
	push @{$db->{recordlist}},"_3:content_data";
    }

    $db->{action} = "insert";

    my $do_out = $db->do;

    if ($db->errorcheck > 0) {
	return ($do_out);
    }
    return ();
    
}

########################
# template page wizard
########################

# A template page is a page for a template website.
# It uses same screen 1 as a regular page, but has different 
# logic for selecting the body.

sub wiz_templatepage_page2 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %page;
    my $pdata;
    my $id = $in->{page_id};

    # always choose a body

    # list all body templates available
    my (@liblist,$libs,$templates,$sql);
    $libs = $db->fetch_match('page',{type=>"library",section_id=>$section});
    $out .= "<p>This page appears to have a conventional layout, with a \"body\" area for holding page-specific content.</p>\n";
    $out .= "<input type=hidden name=content#_2#name value=\"body\">
<input type=hidden name=content#_2#type value=\"editorial\">
<input type=hidden name=content#_2#page_id value=\"$id\">\n";
    $out .= "<p>You can optionally select a pre-defined body for this 
page, from the following list of available body templates.</p>\n<p>";
    foreach my $l (@$libs) {
	push(@liblist, $l->{page_id});
    }
    ######### FIXME: re-write this to avoid a custom query
    if (@$libs > 0) {
	$sql = "select * from content where name='body' and page_id in (".join(',',@liblist).")";
	$templates = $db->custom_query($sql);
    }
    else {
	$templates = [];
    }
    $out .= "<input type=\"radio\" name=\"body_id\" value=\"0\" checked> normal body, fully editable (will be blank to start with)<br>\n";
    foreach my $t (@$templates) {
	$out .= "<input type=\"radio\" name=\"body_id\" value=\"$t->{content_id}\">$t->{description}<br>\n";
    }

    $out .= "<br><br><br>\n";

    return $out;
}

sub do_wiz_templatepage_page2 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};

    my @err = &do_wiz_page_page1($this,$in);
    if (@err > 0) { return @err; }
    
    # now, perform all the necessary DB mods

    # copy selected page body

    if ($in->{body_id}) {
	# substitute the selected body into the form data
	my $c_data = $db->fetch_child("content_data","content",$in->{"body_id"},"content_data_id");
	$c_data = $c_data->[-1];
	$cdata->{note} = "copied from content_data $cdata->{content_data_id}";
	$cdata->{content_id} = "_2";
	delete $cdata->{content_data_id};
	delete $cdata->{ctime};
	$db->{'form'}{'content_data'} = { '_3' => $c_data };
	push @{$db->{recordlist}},"_3:content_data";
    }

    $db->{action} = "insert";

    my $do_out = $db->do;

    if ($db->errorcheck > 0) {
	return ($do_out);
    }
    return ();
    
}

########################
# template wizard
########################

sub wiz_template_page1 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %page;
    my $id = "_1";
    my $type = "template";

    if (! exists $in->{reply}) {
	my $reply = $ENV{HTTP_REFERER} || 
	    "$config{server}{CGIpath}/CMS.cgi$ENV{PATH_INFO}";
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }
    $out .= $db->input_html(name=>"page_id",type=>"hidden",value=>$id);
    $out .= $db->input_html(name=>"page#${id}#section_id",type=>"hidden",value=>$section);
    $out .= $db->input_html(name=>"page#${id}#type",type=>"hidden",value=>$type);
    $out .= $db->input_html(name=>"page#${id}#status",type=>"hidden",value=>"active");
    $out .= "<p>Name:<br>\n";
    $out .= $db->input_column(table=>"page",column=>"title",record=>$id, 
			      data=>\%page);
    $out .= "<p>Description (optional):<br>\n";
    $out .= $db->input_column(table=>"page",column=>"description",record=>$id, 
			      data=>\%page);
    $out .= "<p>Template subdirectory name (alphanumeric characters only; no spaces or periods):<br>\n";
    $out .= $db->input_column(table=>"page",column=>"filename",record=>$id, data=>\%page);
    $out .= "</p>\n";
    return $out;
}

sub do_wiz_template_page1 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};
    my @err;
    if (! $in->{"page#${id}#type"}) {
	push @err, "No page type specified!";
    }
    if (! $in->{"page#${id}#title"}) {
	push @err, "Please specify a title.";
    }
    if (! $in->{"page#${id}#filename"}) {
	push @err, "Please specify a publication directory.";
    }
    if (! $in->{"page#${id}#status"}) {
	push @err, "No status specified!";
    }
    return @err;
}

sub wiz_template_page2 {
    my $this = shift;
    my $in = shift;
    my $out = "<p>A template can itself have a template.  All content in
the parent template will be inherited by the new template.  This allows you
to create numerous variations on a template, without re-importing all the
graphics and stylesheets.  Templates can inherit from other templates to
any depth.</p>\n";
    my %page;
    my $id = $in->{page_id};
    my $type = $in->{"page#${id}#type"};
    
    $out .= "<p>\n";
    $out .= &list_template("page#_1#template_id#key");
    if ($out !~ /\.\.\.hmm,/) {
	# kludge: can't see the no-template error message, so continue
	# prompting for template input
	$out .= "</p>\n<p>If you copy the layout (the \"page\" object) from 
the parent template, you will be able to update/customize the layout for 
this template only.  However, updates to the layout of the parent
template will no longer be inherited by the new template.</p>
<blockquote><p>\n".
           $db->input_html(type=>"checkbox",name=>"copy_layout",selected=>1).
           " Copy Page Layout</p></blockquote>\n";
    }
    return $out;
}

sub do_wiz_template_page2 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};

    # no special validation for this step
    my @err = &do_wiz_template_page1($this,$in);
    if (@err > 0) { return @err; }

    if ($in->{copy_layout} && $in->{'page#_1#template_id#key'}) {
	&wiz_copy_page_layout($this,$in);
    }
    
    $db->{action} = "insert";

    my $do_out = $db->do;

    if ($db->errorcheck > 0) {
	return ($do_out);
    }
    return ();
    
}

# copy the page layout into the form data

sub wiz_copy_page_layout {
    my ($this,$in) = @_;
    if ($in->{'page#_1#template_id#key'}) {
	my $p = new ExSite::Page(id=>$in->{'page#_1#template_id#key'});
	my $pobj = $p->find("page");
	if ($pobj) {
	    # copy the content object
	    my $pobj_data = $pobj->get("content");
	    delete $pobj_data->{content_id}; # force an insert
	    $pobj_data->{page_id} = '_1';
		
	    # copy the content data
	    my $p_data = $pobj->get("content_data");
	    $p_data->{note} = "copied from content_data $p_data->{content_data_id}";
	    delete $p_data->{content_data_id}; # force an insert
	    delete $p_data->{ctime}; 
	    $p_data->{content_id} = '_2';

	    # hack the form data to retroactively insert page object info
	    $db->{form}->{content} = {'_2' => $pobj_data };
	    $db->{form}->{content_data} = {'_3' => $p_data };
	    push @{$db->{recordlist}},"_2:content","_3:content_data";
	}
	else {
	    $this->error("Failed to copy page layout: no page object found!");
	    # log it in the db log, too, since that one is checked more often
	    $db->error("Failed to copy page layout: no page object found!");
	}
    }
}

########################
# library wizard
########################

sub wiz_library_page1 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %page;
    my $id = "_1";
    my $type = "library";

    $out .= "<p>A library is a just a container to hold shareable web content.  
Any page on the site may reference a content item placed in any of that
site's libraries.</p>\n";

    if (! exists $in->{reply}) {
	my $reply = $ENV{HTTP_REFERER} || 
	    "$config{server}{CGIpath}/CMS.cgi$ENV{PATH_INFO}";
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }
    $out .= $db->input_html(name=>"page_id",type=>"hidden",value=>$id);
    $out .= $db->input_html(name=>"page#${id}#section_id",type=>"hidden",value=>$section);
    $out .= $db->input_html(name=>"page#${id}#type",type=>"hidden",value=>$type);
    $out .= $db->input_html(name=>"page#${id}#status",type=>"hidden",value=>"active");
    $out .= "<p>Library Name:<br>\n";
    $out .= $db->input_column(table=>"page",column=>"title",record=>$id, 
			      data=>\%page);
    $out .= "<p>Description (optional):<br>\n";
    $out .= $db->input_column(table=>"page",column=>"description",record=>$id, 
			      data=>\%page);
    $out .= "<p>Library subdirectory name (alphanumeric characters only; no spaces or periods):<br>\n";
    $out .= $db->input_column(table=>"page",column=>"filename",record=>$id, data=>\%page);
    $out .= "</p>\n";
    return $out;
}

sub do_wiz_library_page1 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{page_id};
    my @err;
    if (! $in->{"page#${id}#type"}) {
	push @err, "No page type specified!";
    }
    if (! $in->{"page#${id}#title"}) {
	push @err, "Please specify a title.";
    }
    if (! $in->{"page#${id}#filename"}) {
	push @err, "Please specify a publication directory.";
    }
    if (! $in->{"page#${id}#status"}) {
	push @err, "No status specified!";
    }
    return @err if (@err > 0);

    my $do_out = $db->do;

    if ($db->errorcheck > 0) {
	return ($do_out);
    }
    return ();
}

################################
# website wizard - insert only!
################################

sub wiz_section_page1 {
    my $this = shift;
    my $in = shift;
    my $id = "_1";  # virtual record id
    my %sec;
    my $out = "<p>This wizard can be used to configure entirely new websites, 
or add sub-sections to existing websites.</p>\n";

    # set reply page on first screen
    if (! exists $in->{reply}) {
	my $reply = $ENV{HTTP_REFERER} || 
	    "$config{server}{CGIpath}/CMS.cgi$ENV{PATH_INFO}";
	$out .= $db->input_html(type=>"hidden",name=>"reply",value=>$reply);
    }

    $out .= "<p>Select the website type.<br>\n";

    $out .= $db->input_column(table=>"section",column=>"type",record=>$id,data=>\%sec);
    $out .= "</p>\n<p>Title:<br>\n";
    $out .= $db->input_column(table=>"section",column=>"title",record=>$id,data=>\%sec);
    $out .= "</p>\n<p>Description:<br>\n";
    $out .= $db->input_column(table=>"section",column=>"description",record=>$id,data=>\%sec);
    $out .= $db->input_html(type=>"hidden", name=>"section_id", value=>$id);
    $out .= "</p>\n";
    return $out;
}

sub do_wiz_section_page1 {
    my $this = shift;
    my $in = shift;
    my $id = "_1";  # virtual record id
    my @err;
    if (! ($in->{"section#${id}#type"} && $in->{"section#${id}#title"} && $in->{"section#${id}#description"})) {
	push @err, "Please provide values for all fields.";
    }
    if ($in->{"section#${id}#type"} =~ /template/) {
	# templates have no design, publication setup, or default pages
	$this->{current} += 3 ;
	return $db->do;  # submit form now
    }
    return @err;
}

sub wiz_section_page2 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %sec;
    my $id = "_1";  # virtual record id
#    my $id = $in->{page_id};
    my $type = $in->{"section#${id}#type"};
#    if ($id) {
#	%sec = $db->fetch("section",$in->{page_id});
#    }

    $out .= "<p>If this website or sub-site is related to an existing site, specify the parent site:<br>\n";
    $out .= $db->input_column(table=>"section",column=>"parent_id",record=>$id, data=>\%sec);

    $out .= "</p>\n<p>Menu Label to use if this site is linked to from other sites:<br>";
    $out .= $db->input_column(table=>"section",column=>"label",record=>$id,data=>\%sec);

    $out .= "<p>Specify the publication location ";
    if ($type =~ /remote/) {
	$out .= "as a URL of the form \"ftp://hostname/path/...\":";
    }
    elsif ($type =~ /stand-alone/) {
	$out .= "as a complete disk path to the publication directory:";
    }
    else {
	$out .= "as a subdirectory name (alphanumeric characters and underscores, no spaces):";
    }
    $out .= "<br>\n";
    $out .= $db->input_column(table=>"section",column=>"publish_to",record=>$id, data=>\%sec);
    $out .= "</p>\n";
    if ($type =~ /stand-alone|remote/) {
	$out .= "<p>This site will be accessed publicly from the following domain/URL:<br>\n";
	$out .= $db->input_column(table=>"section",column=>"url",record=>$id, data=>\%sec);
	$out .= "</p>\n";
    }

    return $out;
}

sub do_wiz_section_page2 {
    my $this = shift;
    my $in = shift;
    my $id = "_1";  # virtual record id
#    my $id = $in->{section_id};
    my @err = &do_wiz_section_page1($this,$in);
    my $type = $in->{"section#${id}#type"};
    
    if ($type =~ /remote/ && 
	$in->{"section#${id}#publish_to"} !~ /^ftp:\/\//) {
	push @err, "Please specify a URL to an FTP location.";
    }
    elsif ($type =~ /stand-alone/ && 
	$in->{"section#${id}#publish_to"} !~ /^\/.+/) {
	push @err, "Please specify a complete file path to publish to.";
    }
    else {
	if (! $in->{"section#${id}#publish_to"}) {
	    # only one site may publish to the root directory
	    my @rootsites = $db->fetch_match("section",{publish_to=>""});
	    if (@rootsites > 0) {
		push @err, "Another website is already publishing to the root directory;  please specify a publication subdirectory using alphanumeric characters and underscores only.";
	    }
	}
	elsif ($in->{"section#${id}#publish_to"} !~ /\w+/) {
	    push @err, "Please specify a publication subdirectory using alphanumeric characters and underscores only.";
	}
    }

    return @err;
}

sub wiz_section_page3{
    my $this = shift;
    my $in = shift;
    my $out;
    my %sec;
    my $pdata;
    my $id = "_1";  # virtual record id
#    my $id = $in->{section_id};

    %sec = $db->fetch("section",$id);

    $out .= "<p>You can select a pre-configured set of pages to populate
this website with.  If you do not select a set of pages, the website
will contain no pages to begin with, and you will have to create each
page manually.</p>\n<blockquote>\n";

    my @wtempl = $db->fetch_match("section","type=\"website template\" and (parent_id=0 or parent_id is NULL)");

    if (@wtempl > 0) {
	$out .= "<p><dl>";
	foreach my $wt (@wtempl) {
	    $out .= "<dt>".
		$db->input_html(type=>"radio",name=>"website_template",
				value=>$wt->{section_id},
				label=>"<b>$wt->{title}</b></dt>\n<dd>$wt->{description}</dd>\n");
	}
	$out .= "</dl></p>\n";
    }
    $out .= &list_template("page_template");
    return $out;
}

sub do_wiz_section_page3 {
    my $this = shift;
    my $in = shift;
    # no special validation for this step
    return &do_wiz_section_page2($this,$in);
}

sub wiz_section_page4 {
    my $this = shift;
    my $in = shift;
    my $out;
    my %sec;
    my $pdata;
    my $id = $in->{section_id};

    %sec = $db->fetch("section",$id);

    $out .= "<p>You may optionally define a webmaster for this site.</p> 
<p>If you leave the webmaster undefined, this site will use the
webmaster of its parent site.  If you define a new webmaster, this site 
will function semi-independently of the parent site.</p>\n";

    $out .= "<table cellspacing=20><tr><td width=50% valign=top>
<H4>Method 1: Define a New Webmaster</h4>\n";

    $out .= "<p>\n";
    $db->{action} = "insert";  # dummy, to get input_record to work
    $out .= $db->input_record({table=>"member",record=>"_2",data=>{access=>3},hide=>["access"]});
    #$out .= "<p>Priveleges<br>";
    $out .= $db->input_record({table=>"member_site_link",record=>"_3",data=>{section_id=>"_1",member_id=>"_2",access=>3},hide=>["section_id","member_id"]});
#    $out .= $db->input_exsite(name=>"member_site_link#_1#priveleges",datatype=>"set:member_priv");
    $out .= "</p>\n";

    $out .= "</td><td width=50% valign=top>
<H4>Method 2: Use an Existing Webmaster</h4>\n";

    $out .= $db->input_exsite(name=>"webmaster",datatype=>"key:member",fmatch=>{access=>3});

    #$out .= "<p>Priveleges<br>";
    $out .= $db->input_record({table=>"member_site_link",record=>"_4",data=>{section_id=>"_1",member_id=>"_2",access=>3},hide=>["section_id","member_id"]});
    #$out .= $db->input_exsite(name=>"member_site_link#_2#priveleges",datatype=>"set:member_priv");

    $out .= "<p>This will not affect this webmaster's responsibilities for other web sites.</p>\n";

    $out .= "\n</td></tr></table>\n";

    $out .= "<p>NOTE: if you use both of the above methods, both webmasters will be created.</p>\n";

    return $out;
}

sub do_wiz_section_page4 {
    my $this = shift;
    my $in = shift;
    my $id = $in->{section_id};
    my @err;
    my $new_webmaster;

    # validate all previous inputs

    my @err = &do_wiz_section_page3($this,$in);

    # check new webmaster fields

    if ($in->{"member#_2#login"} || $in->{"member#_2#password"} || $in->{"member#_2#name"} || $in->{"member#_2#email"}) {
	if (! ($in->{"member#_2#login"} && $in->{"member#_2#password"} && $in->{"member#_2#name"} && $in->{"member#_2#email"})) {
	    push @err, "Missing fields for new webmaster.  If not creating a new webmaster, then leave all fields blank.";
	}
	elsif ($in->{"member#_2#email"} !~ /^[\w](\.?[\w\-]+)*@([\w\-]+\.)+[A-Za-z]{2,4}$/) {
	    push @err, "Invalid email address format for new webmaster.";
	}
	else {
	    $new_webmaster = 1;  # the new webmaster form has been filled out
	    my @wmdata = $db->fetch_match("member",{login=>$in->{"member#_2#login"}});
	    if (@wmdata > 0) {
		push @err,"Sorry, a user with that login already exists;  please choose another.";
	    }
	}
    }

    if (@err > 0) { return @err; }
    
    # Looks like we're good to go...
    
    # 1) load up the form data
    $db->get_data;

    # 2) delete any webmaster forms that aren't used
    #print "Content-type: text/plain\n\n";
    if (! $new_webmaster) {
	$db->delete_record("member","_2") or push @err, "failed to ignore new webmaster form";
	$db->delete_record("member_site_link","_3") or push @err, "failed to ignore new webmaster priveleges form";
    }
    if (! $in->{'webmaster#key'}) {
	$db->delete_record("member_site_link","_4") or push @err, "failed to ignore old webmaster priveleges form";
    }
    else {
	# add the existing webmaster to the member_site_link
	$db->{form}{member_site_link}{_4}{member_id} = $in->{'webmaster#key'};
    }

    # delete the webmaster#key form input
    # KLUDGE - use delete_record

    $db->delete_record("webmaster","key") or push @err, "failed to remove old webmaster from form";

    if (@err > 0) { return @err; }

    # insert the basic website records
    my $stat = $db->do;
    if ($stat && $stat !~ /Done!/) {  
	# yuck!  need better error management for doform
	push @err, $stat;
	return @err;
    }

    # now create the default pages
    if ($in->{website_template}) {
	push @err, "using website $in->{website_template}";
	my @pages = $db->fetch_match("page",{section_id=>$in->{website_template}});
	if (@pages > 0) {
	    foreach my $page (@pages) {
		push @err, "making page $page->{page_id}";
		my %newpage = %$page;
		delete $newpage{page_id};  # force an insert
		$newpage{template_id} = $in->{page_template};
		$newpage{section_id} = $db->{form}{section}{_1}{section_id};
		my $id = $db->insert("page",\%newpage);
		if (! $id) {
		    push @err, "failed to add page \"$page->{title}\"";
		}
		else {
		    # make copies of all editorial content objects
		    my @content = $db->fetch_match("content",{page_id=>$page->{page_id},type=>"editorial"});
		    foreach my $cont (@content) {
			delete $cont->{content_id};  # force an insert
			$cont->{page_id} = $id;
			my $cid = $db->insert("content",$cont);
			if (! $cid) {
			    push @err, "failed to add \"$cont->{name}\" content to page \"$page->{title}\"";
			}
		    }
		}
	    }
	}
    }
    return @err;
}

# create a form with a list of available templates

sub changetemplate{
	my ($type) = @_;
	my ($sel,$out,$reply,$templid,$path,$itab,$iid,$icol,$selid);
	$reply = $type ? &goto(@cmspath[1]) : &goto(@cmspath[1..2]);
	$path = $section;
	$path .= $type ? "" : "/$page";
	$sel = $type ? $db->fetch('section',$section) : $db->fetch('page',$page);
	$itab = $type ? 'section' : 'page';
	$iid = $type ? $section : $page;
	$icol = $type ? 'section_id' : 'page_id';
	$selid = $type ? "" : $sel->{template_id};
	$out .= "
	<form method=\"post\" action=\"$CGI/CMS.cgi/$path?action=updatetemplate$type\">\n
	<input type=\"hidden\" name=\"$itab#$iid#$icol\" value=\"$iid\">\n
	<input type=\"hidden\" name=\"$itab#$iid#title\" value=\"$sel->{title}\">";
	$out .= &list_template("$itab#$iid#template_id",$selid);
	$out .= "<input type=\"hidden\" name=\"replyurl\" value=\"$reply\">\n";
	$out .= "<p><input type=\"submit\" value=\" Submit \">\n"; 
	$out .= "<input type=\"button\" value=\" Cancel \" onClick=\"history.back()\">\n</p>"; 
	$out .= "</form>";
	return $out;
}

# updates the database with the new body template

sub updatetemplate{
	my ($type) = @_;
    	my ($out,$do_out,$reply,$title,$itab,$iid,$icol,$allpages,$tempid);
	$title = "page template";
	$title .= $type ? "for all pages" : "";
	$itab = $type ? 'section' : 'page';
	$iid = $type ? $section : $page;
	$icol = $type ? 'section_id' : 'page_id';
	$reply = $type ? &goto(@cmspath[1]) : &goto(@cmspath[1..2]);
	$tempid = $formdata{"$itab#$iid#template_id"};
    	# now, perform all the necessary DB mods
    	$db->get_data;
    	if (!$tempid)
    	{
    		$out .=  "<p>Error: You need to select a $title</p>\n";
		$out .= "<p><form><input type=\"button\" value=\"Back\" onClick=\"history.back()\"></form></p>\n";
		return $out;
    	}
	if ($type){$do_out = $db->update('page',{template_id=>$tempid},{section_id=>$iid,type=>"page"});}
	else
	{
    		$db->{action} = "insert";
    		$do_out = $db->do;
	}
    	if ($db->errorcheck > 0){return ($do_out);}
    	$out .= "<p>Update $title successfully</p>\n";
    	$out .= "<p><form method=\"post\" action=\"$CGI/CMS.cgi\">\n
    		<input type=\"hidden\" name=\"reply\" value=\"$reply\">\n
    		<input type=\"submit\" name=\"submit_action\" value=\"Finish\"></p>\n";
    	return $out;
}

# creates a form with a list of available body templates

sub changebody{
	my ($type,$id,$out,$reply);
	$reply = &goto(@cmspath[1..2]);
	$type = "body content";
	$id = $db->fetch_match('content',{'page_id'=>$page,'name'=>'body'},{'content_id'=>'desc'});
	$id = $id->[0];
	$id = $id->{content_id};
	$out .= "<h3>Change Page Body</h3>
<p>The <b>body</b> is the primary editable section of the page.  This 
tool lets you swap in pre-configured page bodies, if any are available 
for your website to use.</p>
<p>Note that if you simply want to edit or customize your page body, 
you should use the update function.</p>
<p>Also note that some pages may have structures that are too complicated
to be contained in a simple body;  with these pages, changing the body
may only affect part of the editable content, or may even have no effect 
at all if the page doesn't use a body element.</p>
<p>The following pre-configured page bodies are available:</p>\n<p>";
	$out .= "
	<form method=\"post\" action=\"$CGI/CMS.cgi/$section/$page?action=updatebody\">\n
	<input type=\"hidden\" name=\"content_data#_1#content_id\" value=\"$id\">\n";
	my (@liblist,$libs,$templates,$sql);
        $libs = $db->fetch_match('page',{type=>"library",section_id=>$section});
	foreach my $l (@$libs){push(@liblist, $l->{page_id});}
	$sql = "select * from content where name='body' and page_id in (".join(',',@liblist).")";
	$templates = $db->custom_query($sql);
	if (@$templates < 1)
	{
		$out = "<p>There are no body templates available.<br>";
		$out .= "<form><input type=\"button\" value=\"Back\" onClick=\"history.back()\"></form></p>";
		return $out;
	}
	foreach my $t (@$templates)
	{
		$out .= "<input type=\"radio\" name=\"body_id\" value=\"$t->{content_id}\"";
		$out .= ">$t->{description}<br>\n";
	}
	$out .= "<input type=\"hidden\" name=\"content_data#_1#mime_type\" value=\"text/html\">\n";
	$out .= "<input type=\"hidden\" name=\"replyurl\" value=\"$reply\">\n";
	$out .= "<p><input type=\"submit\" value=\" Submit \">\n"; 
	$out .= "<input type=\"button\" value=\" Cancel \" onClick=\"history.back()\">\n</p>"; 
	$out .= "</form>";
	return $out;
}

# updates the database with the new body template

sub updatebody{
    	my ($out,$do_out,$reply);
    	$reply = &goto(@cmspath[1..2]);
    	# now, perform all the necessary DB mods
    	$db->get_data;
    	if (!$formdata{body_id})
    	{
    		$out .=  "<p>Error: You need to select a body template</p>\n";
		$out .= "<p><form><input type=\"button\" value=\"Back\" onClick=\"history.back()\"></form></p>\n";
		return $out;
    	}
    	if ($formdata{body_id})
    	{
		# substitute the selected body into the form data
		my $c_data = $db->fetch_child("content_data","content",$formdata{"body_id"},"content_data_id");
		$c_data = $c_data->[-1];
		$db->{'form'}->{'content_data'}->{'_1'}->{'content_id'} = $formdata{"content_data#_1#content_id"};
		foreach my $c (keys %$c_data)
		{
	    		next if $c =~ /^(content_data_id|content_id|ctime)$/;
	    		$db->{'form'}->{'content_data'}->{'_1'}->{"$c"} = $c_data->{"$c"};
		}
    	}
    	$db->{action} = "insert";
    	$do_out = $db->do;
    	if ($db->errorcheck > 0){return ($do_out);}
    	$out .= "<p>Update body template successfully</p>\n";
    	$out .= "<p><form method=\"post\" action=\"$CGI/CMS.cgi\">\n
    		<input type=\"hidden\" name=\"reply\" value=\"$reply\">\n
    		<input type=\"submit\" name=\"submit_action\" value=\"Finish\"></p>\n";
    	return $out;
}

# sub list_template: create a list of available templates

sub list_template {
    # $input_name: name of the input field
    # $id: template id
    my ($input_name,$id) = @_;
    my ($out,@ptempl,@options);
    $out .= "</blockquote>\n<p>Select a template:</p>\n<blockquote>\n";
    ##### should select only from section path 
    @ptempl = $db->fetch_match("page",{type=>"template"});
    if (@ptempl > 0) {
	foreach my $pt (@ptempl) {
	    $out .= "<p><dl><dt><input type=\"radio\" name=\"$input_name\" value=\"$pt->{page_id}\"";
	    $out .= ($pt->{page_id} == $id) ? " checked" : "";
	    $out .= "> <b>$pt->{title}</b></dt>\n";
	    $out .= "<dd>$pt->{description}  [<A HREF=\"$config{server}->{CGIpath}/page.cgi?_id=$pt->{page_id}\" ";
	    $out .= "TARGET=\"preview\">preview</A>]</dd></dl>\n";
	    # push @options, [$pt->{page_id}, $pt->{title}]; # used for input_html function
	}
    }
    else {
	$out .= "<p>...hmm, there don't seem to be any templates to choose from.  This is not a problem;  just proceed normally.</p>\n"; 
    }
    $out .= "</blockquote>\n";
    return $out;
}
