package Modules::CMS;
#----------------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#-----------------------------------------------------------------------

# CMS : tools for inlining CMS functions into the website itself

use strict;
use ExSite::Config;
use Modules::BaseDCD;

use vars qw(@ISA);
@ISA = qw(Modules::BaseDCD);

sub write {
    my ($this,$options) = @_;
    if ($options =~ /menu/) {
	return $this->show_top_menu;
    }
}

# ioctl : all operations that don't qualify as reads or writes
#         invoked as <!--BaseDCD.ioctl(request)-->

sub ioctl {
    my $this = shift;
    $_ = shift;   # $_ is the ioctl request
    if (/ModuleName/) {
	return "Website Editor";
    }
    elsif (/ControlPanel/) {
	return "$config{server}{CGIpath}/CMS.cgi";
    }
}


sub show_top_menu {
    my $this = shift;
    my $out;
    # page.cgi stores the page record in %share for our use
    my $page_id = $share{Page}->id;
    my $section_id = $share{Page}->id("section");
    my $JS = "$config{server}{HTMLpath}/_ExSite/js";
    my $CGI = "$config{server}{CGIpath}";
    my $HTML = "$config{server}{HTMLpath}";

    # add CBE menu stylesheet
    $out .= "<LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"$JS/menu.css\">\n";

    # add JavaScript libraries
    $out .= "<script type='text/javascript' src='$JS/cbe_core.js'></script>
<script type='text/javascript' src='$JS/cbe_event.js'></script>
<script type='text/javascript' src='$JS/cbe_util.js'></script><!-- for debugging -->
<script type='text/javascript' src='$JS/cbe_debug.js'></script><!-- for debugging -->
<script type='text/javascript' src='$JS/menu_exsite.js'></script>

<script type='text/javascript'><!--
var mark1, menu1, menu2, menu3;
function windowOnload() {
  mark1 = cbeGetElementById('menuMarker1').cbe;
  menu1 = new cbeMenu(
    true,                       // horizontal,
//    (document.cbe.width()-(3*75))/2, mark1.pageY(), 3, // centered
    mark1.pageX(), mark1.pageY(), 3, // menuX, menuY, menuZ (left-justified)
    [75,75,75], 20, 8,          // lblWidthsArray, lblHeight, lblspacing
    [140,170,155],              // boxWidthsArray,
    '#000099', '#000000',       // activeColor, inactiveColor,
    '#FFFFFF', '#C0C0C0',       // activeBkgnd, inactiveBkgnd
    '#C0C0C0'                   // boxBkgnd
  );
  window.cbe.addEventListener(\"resize\", resizeListener, false);
}

--></script>\n";

    # now build the HTML for the menubar

    $out .= "<a name=\"topofpage\"></a>
 <TABLE WIDTH=100% CELLSPACING=0 CELLPADDING=0 BORDER=0> 
  <TR>
   <TD BGCOLOR=#DDDDDD COLSPAN=4><IMG SRC=null.gif HEIGHT=1 WIDTH=1></TD>
  </TR>
   <TR>
   <TD BGCOLOR=#DDDDDD><IMG SRC=null.gif HEIGHT=1 WIDTH=1></TD>
   <TD BGCOLOR=#C0C0C0>
     &nbsp;&nbsp;
   </TD>
   <TD BGCOLOR=#C0C0C0 WIDTH=100%>
    <div id='menuMarker1' class='menuMarker'>&nbsp;</div>
   </TD>
   <TD BGCOLOR=#666666><IMG SRC=null.gif HEIGHT=1 WIDTH=1></TD>
  </TR>
  <TR>
   <TD BGCOLOR=#666666 COLSPAN=4><IMG SRC=null.gif HEIGHT=1 WIDTH=1></TD>
  </TR>
 </TABLE>\n";

    # now create the menus...

    # menu 1 : website manager

    $out .= "<div id='label11' class='mLabel'>Website</div>
<div id='box11' class='mBox'>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id?action=map2page\">Site Map</a>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id\">Advanced Tools</a>
  <a class=\"m\" href=\"$CGI/publish.cgi?section=$section_id\" target=publish>Publish Site</a>
</div>\n";

    # menu 2 : page manager

    ########### view published link will fail for non-home sections

    $out .= "<div id='label12' class='mLabel'>Page</div>
<div id='box12' class='mBox'>
  <a class=\"m\" href=\"$HTML/$share{Page}->{page}->{filename}\">View Published</a>
  <a class=\"m\" href=\"$CGI/page.cgi?_id=$page_id\">View Current</a>
  <a class=\"m\" href=\"$CGI/page.cgi?_id=$page_id&_hot=1&_cms=1\">View Editable</a>
  <hr>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id/$page_id?action=changebody\">Change Body</a>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id/$page_id?action=editpage\">Page Settings</a>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id/$page_id\">Advanced Tools</a>
  <hr>
  <a class=\"m\" href=\"$CGI/publish.cgi?page=$page_id\" target=publish>Publish Page</a>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id/$page_id?action=newpage\">New Page</a>
  <a class=\"m\" href=\"$CGI/CMS.cgi/$section_id/$page_id?action=deletepage\">Delete Page</a>
</div>\n";

    # menu 3 : help

    $out .= "<div id='label13' class='mLabel'>Help</div>
<div id='box13' class='mBox'>
  <a class=\"m\" href=\"javascript:alert('Help not enabled.')\">Help</a>
  <a class=\"m\" href=\"javascript:alert('About not enabled.')\">About</a>
  <hr>
  <a class=\"m\" target=\"log\" href=\"$CGI/exsite_log.cgi\">View System Logs</a>
</div>\n";

    # menu 4 : exit

    $out .= "<div id='label14' class='mLabel'>Exit</div>
<div id='box14' class='mBox'>
  <a class=\"m\" href=\"$CGI/home.cgi\">Close</a>
  <a class=\"m\" href=\"$CGI/page.cgi?_id=$page_id&_cms=0\">Turn off CMS - this page</a>
  <a class=\"m\" href=\"$CGI/page.cgi?_id=$page_id&_cmsmode=0\">Turn off CMS - all pages</a>
  <a class=\"m\" href=\"$CGI/logout.cgi\">Log out</a>
</div>\n";

    return $out;
}

1;
