#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_dyncontent.cgi: custom dialog used by HTML editor for adding 
# dynamic content
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;
use ExSite::Content;
use myConfig; 

use vars qw(%data $uid $access $OUT $operation $db $type);

&exsite_init;
$config{page_buffering} = 0;
%data = &DecodeInput;  # accepts input by post or query
$db = new ExSite::DB;
$access = $db->authorize;
$operation = ($data{url}) ? "Edit" : "Insert";
$type = "Web Application";

if ($access==0) {
	print "Location: login.cgi\n\n";
} elsif ($access<2) {
	$OUT .= &PrintHeader;
	$OUT .= "<p>Access denied.  Administrators only.</p>";
	$OUT .= &PrintFooter;
	print $OUT;
} else {
	$OUT .= &PrintHeader;	
	$OUT .= &PrintForm;
	$OUT .= &PrintFooter;
	print $OUT;
}

#---------------------------------------------------------------------------

sub PrintHeader {
	my (@links) = @_;
	my $out = "Content-type: text/html\n\n
<HTML>
<HEAD>
<TITLE>$operation $type</TITLE>";
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= "<style>
	BODY { background-color: buttonface; }
	TD, INPUT, SELECT { font-family: \"MS Sans Serif\"; font-size: xx-small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button { border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }
</style>";
	} else {	
		$out .= "<style>
	BODY { background-color: buttonface; }
	TD, INPUT, P { font-family: sans-serif; font-size: small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px; }
</style>";
	}
	$out .= "\n<script language=\"JavaScript\">
<!--
	function insertdyncontent() {
		var name,param;
		name = document.forms[0].dyncontent.value;
		param = document.forms[0].param.value;
		window.opener.getDynContent(name,param);
		window.close();
	}
	function cancel() {
		window.returnValue = null;
		window.close();
	}";
	$out .= "
//-->
</script>
</head>
<body topmargin=\"0\" leftmargin=\"0\" style=\"border: 0; margin: 0;\" scroll=\"no\">\n";
	return $out;
}

sub PrintFooter {
	return "\n</body></html>";
}

sub PrintForm
{
        my (@list,$ct);
	$ct = new ExSite::Content;
	@list = $ct->get_dcd;
	return "<p>Sorry, no web applications are available.</p>" if (scalar(@list) < 1);
        my $out = "<form onSubmit=\"insertdyncontent()\">\n";
        $out .= "<table border=\"0\">\n";
        $out .= "<tr><td colspan=\"2\">\n<b>Insert Web Application</b><br><br>\n</td></tr>\n";
	$out .= "<tr><td><b>Web Application: </b></td><td>";
	$out .= "<select name=\"dyncontent\">";
	$out .= "<option value=\"\">== select ==</option>";
	foreach my $l (@list){
		my $name = $l;
		$name =~ s/\.pm$//g;
		$out .= "<option value=\"$name\">$name</option>";
	}
	$out .= "</select>";
	$out .= "</td></tr>\n";
	$out .= "<tr><td width=\"100\"><b>Parameter (optional): </b></td>";
	$out .= "<td><input type=\"text\" name=\"param\" value=\"\"></td></tr>\n";	
        $out .= "<tr><td>";
        $out .= "\n<input type=\"button\" value=\"Insert\" onClick=\"insertdyncontent();\">";
        $out .= "\n</td><td>";
        $out .= "\n<input type=\"button\" value=\"Cancel\" onClick=\"cancel();\">";
        $out .= "\n</td></tr>";
        $out .= "\n</table>\n";
        $out .= "</form>\n";
	return $out;
}

### END
