#!/usr/bin/perl
#-----------------------------------------------------------------------
# dlg_image.cgi: custom dialog used by HTML editor for adding and editing 
# anchor tags
#-----------------------------------------------------------------------
#
#   Copyright 2001-2004 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::DB;
use ExSite::Content;
use ExSite::Section;
use ExSite::Page;
use myConfig; 

use vars qw(%data $uid $access $OUT $operation $db $type);

my $debug = 0;
print "content-type: text/html\n\n" if $debug;

&exsite_init;
$config{page_buffering} = 0;
%data = &DecodeInput;  # accepts input by post or query
$db = new ExSite::DB;
$access = $db->authorize;
$operation = ($data{url}) ? "Edit" : "Insert";
$type = "Image";

if ($access==0) {
	print "Location: login.cgi\n\n";
} elsif ($access<2) {
	$OUT .= &PrintHeader;
	$OUT .= "<p>Access denied.  Webmasters only.</p>";
	$OUT .= &PrintFooter;
	print $OUT;
} else {
	my @links = &GetImageLinks;
	$OUT .= &PrintHeader(@links);	
	$OUT .= &PrintForm(@links);
	$OUT .= &PrintFooter;
	print $OUT;
}
#--------------------------------------------------------------------------------------
sub GetImageLinks {
	my (@links,$page_list,$page,$sql,@clist,$co,$section,$cdata,@newlist);
	if (exists $data{data_id}) { 
	    $cdata = new ExSite::ContentData(id=>$data{data_id});
	    $cdata->set_context();
	    $section = $cdata->get("section");   # fetches section data as a hash ref 

	}
	else {
	    $cdata = new ExSite::Page(id=>$data{cont_id});
	    $cdata = $cdata->get;
	    $section = $db->fetch('section',$cdata->{section_id});
	}	
	$co = new ExSite::Section(id=>$section->{section_id});
	@clist = $co->fetch_library_content(allow_mime=>"image");
	foreach my $c (@clist){
		$c = $c->get;
		push(@newlist,$c);
	}
	return @newlist;
}

sub PrintHeader {
	my (@links) = @_;
	my $out = "Content-type: text/html\n\n
<HTML>
<HEAD>
<TITLE>$operation $type</TITLE>";
	if ($ENV{'HTTP_USER_AGENT'} =~ /msie/i){
		$out .= "<style>
	BODY { background-color: buttonface; }
	TD, INPUT, SELECT { font-family: \"MS Sans Serif\"; font-size: xx-small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button { border-top: 1px solid white; border-left: 1px solid white;
		border-bottom: 1px solid black; border-right: 1px solid black;
		font-size: x-small; width: 60px; }
	SELECT { height: 75%; }
</style>";
	} else {	
		$out .= "<style>
	BODY { background-color: buttonface; }
	TD, INPUT, P { font-family: sans-serif; font-size: small; vertical-align: middle; }
	TABLE.dlg { border:0; }
	.dlg TD { horizontal-align: left; height: 20px; }
	.dlg INPUT { border-size: 2px; }
	INPUT.button_style { 
		border-top: 1px solid white; 
		border-left: 1px solid white;
		border-bottom: 1px solid black; 
		border-right: 1px solid black;
		font-size: x-small; 
		width: 60px; }
</style>";
	}
	$out .= "\n<script language=\"JavaScript\">
<!--
	function insertimage() {
		var name,align;
		name = document.forms[0].id.value;
		for(var i = 0; i < 3; i++){
			if(document.forms[0].align[i].checked){
				align = document.forms[0].align[i].value;
			}
		}
		window.opener.getImage(name,align);
		window.close();
	}
	function cancel() {
		window.returnValue = null;
		window.close();
	}";
	$out .= "
	function showimage() {
		var target,url,id,image;
		target = target = document.getElementById('disp');
		url = new Object();";
	foreach my $l (@links){
		my ($c,$img);
		$c = new ExSite::Content(name=>"$l->{name}",page=>$l->{page_id},use=>"active");
		$img = $c->get_html;
		$out .= "\n\t\turl['$l->{name}'] = '".substr($img,0,length($img)-1)."' + ' width=\"100\">';";
	}
	$out .= "
		id = document.forms[0].id.value;
		if (url[id]) {
			image = url[id];
		} else {
			image = '';
		}
		target.innerHTML = image;
	}";
	$out .= "
//-->
</script>
</head>
<body topmargin=\"0\" leftmargin=\"0\" style=\"border: 0; margin: 0;\" scroll=\"no\">\n";
	return $out;
}

sub PrintFooter {
	return "\n</body></html>";
}

sub PrintForm
{
        my (@list) = @_;
	return "<p>sorry, no image is available.</p>" if (scalar(@list) < 1);
        my $out = "<form onSubmit=\"insertimage()\">\n";
        $out .= "<table border=\"0\">\n";
        $out .= "<tr><td colspan=\"2\">\n<b>Select an image to insert into the editor</b>\n</td></tr>\n";
        $out .= "<tr><td valign=\"top\" width=\"150\">\n<b>Image Preview</b><br><br>";
        $out .= "<div id=\"disp\"></div>";
        $out .= "\n</td><td valign=\"top\">";
        $out .= "\n<b>Select an image:</b><br>";
        $out .= "\n<select name=\"id\" onClick=\"showimage();\">";
        $out .= "\n<option value=\"\">None</option>\n";
        foreach my $l (@list){$out .= "<option value=\"$l->{name}\">$l->{name}</option>\n";}
        $out .= "</select><br><br>\n";
        $out .= "\n<b>Image Alignment:</b><br>";
        $out .= "\n<input type=\"radio\" name=\"align\" value=\"\" checked>None<br>";
        $out .= "\n<input type=\"radio\" name=\"align\" value=\"left\">Left<br>";
        $out .= "\n<input type=\"radio\" name=\"align\" value=\"right\">Right<br>";
        $out .= "</td></tr>";
        $out .= "<tr><td>";
        $out .= "\n<input type=\"button\" value=\"Insert Image\" onClick=\"insertimage();\">";
        $out .= "\n</td><td>";
        $out .= "\n<input type=\"button\" value=\"Cancel\" onClick=\"cancel();\">";
        $out .= "\n</td></tr>";
        $out .= "\n</table>\n";
        $out .= "</form>\n";
	return $out;
}

sub PrintForm_old {
	my (@links) = @_;
	my $out = "
<table border=\"0\" class=\"dlg\" cellpadding=\"0\" cellspacing=\"2\" width=\"100%\">
<tr><td width=\"100\">Insert $type</td><td align=\"left\" valign=\"middle\"><hr width=\"100%\"></td></tr>
</table>";
	if (@links)
	{
		$out .= "<form>
<table border=\"0\" class=\"dlg\" cellpadding=\"0\" cellspacing=\"2\" width=\"100%\" height=\"100%\">
<tr>\n";
		foreach my $l (@links){$out .= "<td><a href=\"#\" onclick=\"insertimage('$l->{name}')\">$l->{name}</a></td>\n";}
		$out .= "</tr>
</table>
</form>";
	}
	else{$out .= "<p>There are no $type"."s in the database.</p>";}
	$out .= "<table border=\"0\" width=\"100%\">\n<tr>\n<td align=\"right\">\n";
	$out .= "<form><input class=\"button\" type=\"button\" value=\"Cancel\" onClick=\"window.close()\"></form>\n";
	$out .= "</td>\n</tr>\n</table>\n";
	return $out;
}
### END
